@@ -47,35 +47,73 @@ import java.io.OutputStream;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 
 import org.apache.commons.compress.archivers.ArchiveOutputStream;
 import org.apache.commons.compress.compressors.xz.XZCompressorOutputStream;
 import org.eclipse.jgit.api.ArchiveCommand;
 import org.eclipse.jgit.lib.FileMode;
+import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.ObjectLoader;
 
 /**
  * Xz-compressed tar (tar.xz) format.
  */
-public class TxzFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
+public final class TxzFormat extends BaseFormat implements
+		ArchiveCommand.Format<ArchiveOutputStream> {
 	private static final List<String> SUFFIXES = Collections
 			.unmodifiableList(Arrays.asList(".tar.xz", ".txz")); //$NON-NLS-1$ //$NON-NLS-2$
 
 	private final ArchiveCommand.Format<ArchiveOutputStream> tarFormat = new TarFormat();
 
+	/** {@inheritDoc} */
+	@Override
 	public ArchiveOutputStream createArchiveOutputStream(OutputStream s)
 			throws IOException {
-		XZCompressorOutputStream out = new XZCompressorOutputStream(s);
-		return tarFormat.createArchiveOutputStream(out);
+		return createArchiveOutputStream(s,
+				Collections.<String, Object> emptyMap());
 	}
 
+	/** {@inheritDoc} */
+	@Override
+	public ArchiveOutputStream createArchiveOutputStream(OutputStream s,
+			Map<String, Object> o) throws IOException {
+		XZCompressorOutputStream out = new XZCompressorOutputStream(s);
+		return tarFormat.createArchiveOutputStream(out, o);
+	}
+
+	/** {@inheritDoc} */
+	@Deprecated
+	@Override
 	public void putEntry(ArchiveOutputStream out,
 			String path, FileMode mode, ObjectLoader loader)
 			throws IOException {
-		tarFormat.putEntry(out, path, mode, loader);
+		putEntry(out, null, path, mode,loader);
 	}
 
+	/** {@inheritDoc} */
+	@Override
+	public void putEntry(ArchiveOutputStream out,
+			ObjectId tree, String path, FileMode mode, ObjectLoader loader)
+			throws IOException {
+		tarFormat.putEntry(out, tree, path, mode, loader);
+	}
+
+	/** {@inheritDoc} */
+	@Override
 	public Iterable<String> suffixes() {
 		return SUFFIXES;
 	}
+
+	/** {@inheritDoc} */
+	@Override
+	public boolean equals(Object other) {
+		return (other instanceof TxzFormat);
+	}
+
+	/** {@inheritDoc} */
+	@Override
+	public int hashCode() {
+		return getClass().hashCode();
+	}
 }
