@@ -43,21 +43,26 @@
 
 package org.eclipse.jgit.junit;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.fail;
 
+import java.io.BufferedOutputStream;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.security.MessageDigest;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.TimeZone;
 
+import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.dircache.DirCacheBuilder;
 import org.eclipse.jgit.dircache.DirCacheEditor;
@@ -87,6 +92,8 @@ import org.eclipse.jgit.lib.RefUpdate;
 import org.eclipse.jgit.lib.RefWriter;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.TagBuilder;
+import org.eclipse.jgit.merge.MergeStrategy;
+import org.eclipse.jgit.merge.ThreeWayMerger;
 import org.eclipse.jgit.revwalk.ObjectWalk;
 import org.eclipse.jgit.revwalk.RevBlob;
 import org.eclipse.jgit.revwalk.RevCommit;
@@ -96,8 +103,8 @@ import org.eclipse.jgit.revwalk.RevTree;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.jgit.treewalk.filter.PathFilterGroup;
+import org.eclipse.jgit.util.ChangeIdUtil;
 import org.eclipse.jgit.util.FileUtils;
-import org.eclipse.jgit.util.io.SafeBufferedOutputStream;
 
 /**
  * Wrapper to make creating test data easier.
@@ -106,31 +113,32 @@ import org.eclipse.jgit.util.io.SafeBufferedOutputStream;
  *            type of Repository the test data is stored on.
  */
 public class TestRepository<R extends Repository> {
-	private static final PersonIdent author;
 
-	private static final PersonIdent committer;
+	/** Constant <code>AUTHOR="J. Author"</code> */
+	public static final String AUTHOR = "J. Author";
 
-	static {
-		final MockSystemReader m = new MockSystemReader();
-		final long now = m.getCurrentTime();
-		final int tz = m.getTimezone(now);
+	/** Constant <code>AUTHOR_EMAIL="jauthor@example.com"</code> */
+	public static final String AUTHOR_EMAIL = "jauthor@example.com";
 
-		final String an = "J. Author";
-		final String ae = "jauthor@example.com";
-		author = new PersonIdent(an, ae, now, tz);
+	/** Constant <code>COMMITTER="J. Committer"</code> */
+	public static final String COMMITTER = "J. Committer";
 
-		final String cn = "J. Committer";
-		final String ce = "jcommitter@example.com";
-		committer = new PersonIdent(cn, ce, now, tz);
-	}
+	/** Constant <code>COMMITTER_EMAIL="jcommitter@example.com"</code> */
+	public static final String COMMITTER_EMAIL = "jcommitter@example.com";
+
+	private final PersonIdent defaultAuthor;
+
+	private final PersonIdent defaultCommitter;
 
 	private final R db;
 
+	private final Git git;
+
 	private final RevWalk pool;
 
 	private final ObjectInserter inserter;
 
-	private long now;
+	private final MockSystemReader mockSystemReader;
 
 	/**
 	 * Wrap a repository with test building tools.
@@ -140,7 +148,7 @@ public class TestRepository<R extends Repository> {
 	 * @throws IOException
 	 */
 	public TestRepository(R db) throws IOException {
-		this(db, new RevWalk(db));
+		this(db, new RevWalk(db), new MockSystemReader());
 	}
 
 	/**
@@ -153,25 +161,81 @@ public class TestRepository<R extends Repository> {
 	 * @throws IOException
 	 */
 	public TestRepository(R db, RevWalk rw) throws IOException {
-		this.db = db;
-		this.pool = rw;
-		this.inserter = db.newObjectInserter();
-		this.now = 1236977987000L;
+		this(db, rw, new MockSystemReader());
 	}
 
-	/** @return the repository this helper class operates against. */
+	/**
+	 * Wrap a repository with test building tools.
+	 *
+	 * @param db
+	 *            the test repository to write into.
+	 * @param rw
+	 *            the RevObject pool to use for object lookup.
+	 * @param reader
+	 *            the MockSystemReader to use for clock and other system
+	 *            operations.
+	 * @throws IOException
+	 * @since 4.2
+	 */
+	public TestRepository(R db, RevWalk rw, MockSystemReader reader)
+			throws IOException {
+		this.db = db;
+		this.git = Git.wrap(db);
+		this.pool = rw;
+		this.inserter = db.newObjectInserter();
+		this.mockSystemReader = reader;
+		long now = mockSystemReader.getCurrentTime();
+		int tz = mockSystemReader.getTimezone(now);
+		defaultAuthor = new PersonIdent(AUTHOR, AUTHOR_EMAIL, now, tz);
+		defaultCommitter = new PersonIdent(COMMITTER, COMMITTER_EMAIL, now, tz);
+	}
+
+	/**
+	 * Get repository
+	 *
+	 * @return the repository this helper class operates against.
+	 */
 	public R getRepository() {
 		return db;
 	}
 
-	/** @return get the RevWalk pool all objects are allocated through. */
+	/**
+	 * Get RevWalk
+	 *
+	 * @return get the RevWalk pool all objects are allocated through.
+	 */
 	public RevWalk getRevWalk() {
 		return pool;
 	}
 
-	/** @return current time adjusted by {@link #tick(int)}. */
-	public Date getClock() {
-		return new Date(now);
+	/**
+	 * Return Git API wrapper
+	 *
+	 * @return an API wrapper for the underlying repository. This wrapper does
+	 *         not allocate any new resources and need not be closed (but
+	 *         closing it is harmless).
+	 */
+	public Git git() {
+		return git;
+	}
+
+	/**
+	 * Get date
+	 *
+	 * @return current date.
+	 * @since 4.2
+	 */
+	public Date getDate() {
+		return new Date(mockSystemReader.getCurrentTime());
+	}
+
+	/**
+	 * Get timezone
+	 *
+	 * @return timezone used for default identities.
+	 */
+	public TimeZone getTimeZone() {
+		return mockSystemReader.getTimeZone();
 	}
 
 	/**
@@ -181,18 +245,18 @@ public class TestRepository<R extends Repository> {
 	 *            number of seconds to add to the current time.
 	 */
 	public void tick(final int secDelta) {
-		now += secDelta * 1000L;
+		mockSystemReader.tick(secDelta);
 	}
 
 	/**
-	 * Set the author and committer using {@link #getClock()}.
+	 * Set the author and committer using {@link #getDate()}.
 	 *
 	 * @param c
 	 *            the commit builder to store.
 	 */
 	public void setAuthorAndCommitter(org.eclipse.jgit.lib.CommitBuilder c) {
-		c.setAuthor(new PersonIdent(author, new Date(now)));
-		c.setCommitter(new PersonIdent(committer, new Date(now)));
+		c.setAuthor(new PersonIdent(defaultAuthor, getDate()));
+		c.setCommitter(new PersonIdent(defaultCommitter, getDate()));
 	}
 
 	/**
@@ -204,7 +268,7 @@ public class TestRepository<R extends Repository> {
 	 * @throws Exception
 	 */
 	public RevBlob blob(final String content) throws Exception {
-		return blob(content.getBytes("UTF-8"));
+		return blob(content.getBytes(UTF_8));
 	}
 
 	/**
@@ -217,11 +281,9 @@ public class TestRepository<R extends Repository> {
 	 */
 	public RevBlob blob(final byte[] content) throws Exception {
 		ObjectId id;
-		try {
-			id = inserter.insert(Constants.OBJ_BLOB, content);
-			inserter.flush();
-		} finally {
-			inserter.release();
+		try (ObjectInserter ins = inserter) {
+			id = ins.insert(Constants.OBJ_BLOB, content);
+			ins.flush();
 		}
 		return pool.lookupBlob(id);
 	}
@@ -260,11 +322,9 @@ public class TestRepository<R extends Repository> {
 			b.add(e);
 		b.finish();
 		ObjectId root;
-		try {
-			root = dc.writeTree(inserter);
-			inserter.flush();
-		} finally {
-			inserter.release();
+		try (ObjectInserter ins = inserter) {
+			root = dc.writeTree(ins);
+			ins.flush();
 		}
 		return pool.lookupTree(root);
 	}
@@ -281,18 +341,19 @@ public class TestRepository<R extends Repository> {
 	 */
 	public RevObject get(final RevTree tree, final String path)
 			throws Exception {
-		final TreeWalk tw = new TreeWalk(pool.getObjectReader());
-		tw.setFilter(PathFilterGroup.createFromStrings(Collections
-				.singleton(path)));
-		tw.reset(tree);
-		while (tw.next()) {
-			if (tw.isSubtree() && !path.equals(tw.getPathString())) {
-				tw.enterSubtree();
-				continue;
+		try (TreeWalk tw = new TreeWalk(pool.getObjectReader())) {
+			tw.setFilter(PathFilterGroup.createFromStrings(Collections
+					.singleton(path)));
+			tw.reset(tree);
+			while (tw.next()) {
+				if (tw.isSubtree() && !path.equals(tw.getPathString())) {
+					tw.enterSubtree();
+					continue;
+				}
+				final ObjectId entid = tw.getObjectId(0);
+				final FileMode entmode = tw.getFileMode(0);
+				return pool.lookupAny(entid, entmode.getObjectType());
 			}
-			final ObjectId entid = tw.getObjectId(0);
-			final FileMode entmode = tw.getFileMode(0);
-			return pool.lookupAny(entid, entmode.getObjectType());
 		}
 		fail("Can't find " + path + " in tree " + tree.name());
 		return null; // never reached.
@@ -373,20 +434,22 @@ public class TestRepository<R extends Repository> {
 		c = new org.eclipse.jgit.lib.CommitBuilder();
 		c.setTreeId(tree);
 		c.setParentIds(parents);
-		c.setAuthor(new PersonIdent(author, new Date(now)));
-		c.setCommitter(new PersonIdent(committer, new Date(now)));
+		c.setAuthor(new PersonIdent(defaultAuthor, getDate()));
+		c.setCommitter(new PersonIdent(defaultCommitter, getDate()));
 		c.setMessage("");
 		ObjectId id;
-		try {
-			id = inserter.insert(c);
-			inserter.flush();
-		} finally {
-			inserter.release();
+		try (ObjectInserter ins = inserter) {
+			id = ins.insert(c);
+			ins.flush();
 		}
 		return pool.lookupCommit(id);
 	}
 
-	/** @return a new commit builder. */
+	/**
+	 * Create commit builder
+	 *
+	 * @return a new commit builder.
+	 */
 	public CommitBuilder commit() {
 		return new CommitBuilder();
 	}
@@ -411,14 +474,12 @@ public class TestRepository<R extends Repository> {
 		final TagBuilder t = new TagBuilder();
 		t.setObjectId(dst);
 		t.setTag(name);
-		t.setTagger(new PersonIdent(committer, new Date(now)));
+		t.setTagger(new PersonIdent(defaultCommitter, getDate()));
 		t.setMessage("");
 		ObjectId id;
-		try {
-			id = inserter.insert(t);
-			inserter.flush();
-		} finally {
-			inserter.release();
+		try (ObjectInserter ins = inserter) {
+			id = ins.insert(t);
+			ins.flush();
 		}
 		return (RevTag) pool.lookupAny(id, Constants.OBJ_TAG);
 	}
@@ -441,6 +502,72 @@ public class TestRepository<R extends Repository> {
 		return update(ref, to.create());
 	}
 
+	/**
+	 * Amend an existing ref.
+	 *
+	 * @param ref
+	 *            the name of the reference to amend, which must already exist.
+	 *            If {@code ref} does not start with {@code refs/} and is not the
+	 *            magic names {@code HEAD} {@code FETCH_HEAD} or {@code
+	 *            MERGE_HEAD}, then {@code refs/heads/} will be prefixed in front
+	 *            of the given name, thereby assuming it is a branch.
+	 * @return commit builder that amends the branch on commit.
+	 * @throws Exception
+	 */
+	public CommitBuilder amendRef(String ref) throws Exception {
+		String name = normalizeRef(ref);
+		Ref r = db.exactRef(name);
+		if (r == null)
+			throw new IOException("Not a ref: " + ref);
+		return amend(pool.parseCommit(r.getObjectId()), branch(name).commit());
+	}
+
+	/**
+	 * Amend an existing commit.
+	 *
+	 * @param id
+	 *            the id of the commit to amend.
+	 * @return commit builder.
+	 * @throws Exception
+	 */
+	public CommitBuilder amend(AnyObjectId id) throws Exception {
+		return amend(pool.parseCommit(id), commit());
+	}
+
+	private CommitBuilder amend(RevCommit old, CommitBuilder b) throws Exception {
+		pool.parseBody(old);
+		b.author(old.getAuthorIdent());
+		b.committer(old.getCommitterIdent());
+		b.message(old.getFullMessage());
+		// Use the committer name from the old commit, but update it after ticking
+		// the clock in CommitBuilder#create().
+		b.updateCommitterTime = true;
+
+		// Reset parents to original parents.
+		b.noParents();
+		for (int i = 0; i < old.getParentCount(); i++)
+			b.parent(old.getParent(i));
+
+		// Reset tree to original tree; resetting parents reset tree contents to the
+		// first parent.
+		b.tree.clear();
+		try (TreeWalk tw = new TreeWalk(db)) {
+			tw.reset(old.getTree());
+			tw.setRecursive(true);
+			while (tw.next()) {
+				b.edit(new PathEdit(tw.getPathString()) {
+					@Override
+					public void apply(DirCacheEntry ent) {
+						ent.setFileMode(tw.getFileMode(0));
+						ent.setObjectId(tw.getObjectId(0));
+					}
+				});
+			}
+		}
+
+		return b;
+	}
+
 	/**
 	 * Update a reference to point to an object.
 	 *
@@ -458,17 +585,7 @@ public class TestRepository<R extends Repository> {
 	 * @throws Exception
 	 */
 	public <T extends AnyObjectId> T update(String ref, T obj) throws Exception {
-		if (Constants.HEAD.equals(ref)) {
-			// nothing
-		} else if ("FETCH_HEAD".equals(ref)) {
-			// nothing
-		} else if ("MERGE_HEAD".equals(ref)) {
-			// nothing
-		} else if (ref.startsWith(Constants.R_REFS)) {
-			// nothing
-		} else
-			ref = Constants.R_HEADS + ref;
-
+		ref = normalizeRef(ref);
 		RefUpdate u = db.updateRef(ref);
 		u.setNewObjectId(obj);
 		switch (u.forceUpdate()) {
@@ -484,6 +601,154 @@ public class TestRepository<R extends Repository> {
 		}
 	}
 
+	/**
+	 * Delete a reference.
+	 *
+	 * @param ref
+	 *	      the name of the reference to delete. This is normalized
+	 *	      in the same way as {@link #update(String, AnyObjectId)}.
+	 * @throws Exception
+	 * @since 4.4
+	 */
+	public void delete(String ref) throws Exception {
+		ref = normalizeRef(ref);
+		RefUpdate u = db.updateRef(ref);
+		u.setForceUpdate(true);
+		switch (u.delete()) {
+		case FAST_FORWARD:
+		case FORCED:
+		case NEW:
+		case NO_CHANGE:
+			updateServerInfo();
+			return;
+
+		default:
+			throw new IOException("Cannot delete " + ref + " " + u.getResult());
+		}
+	}
+
+	private static String normalizeRef(String ref) {
+		if (Constants.HEAD.equals(ref)) {
+			// nothing
+		} else if ("FETCH_HEAD".equals(ref)) {
+			// nothing
+		} else if ("MERGE_HEAD".equals(ref)) {
+			// nothing
+		} else if (ref.startsWith(Constants.R_REFS)) {
+			// nothing
+		} else
+			ref = Constants.R_HEADS + ref;
+		return ref;
+	}
+
+	/**
+	 * Soft-reset HEAD to a detached state.
+	 *
+	 * @param id
+	 *            ID of detached head.
+	 * @throws Exception
+	 * @see #reset(String)
+	 */
+	public void reset(AnyObjectId id) throws Exception {
+		RefUpdate ru = db.updateRef(Constants.HEAD, true);
+		ru.setNewObjectId(id);
+		RefUpdate.Result result = ru.forceUpdate();
+		switch (result) {
+			case FAST_FORWARD:
+			case FORCED:
+			case NEW:
+			case NO_CHANGE:
+				break;
+			default:
+				throw new IOException(String.format(
+						"Checkout \"%s\" failed: %s", id.name(), result));
+		}
+	}
+
+	/**
+	 * Soft-reset HEAD to a different commit.
+	 * <p>
+	 * This is equivalent to {@code git reset --soft} in that it modifies HEAD but
+	 * not the index or the working tree of a non-bare repository.
+	 *
+	 * @param name
+	 *            revision string; either an existing ref name, or something that
+	 *            can be parsed to an object ID.
+	 * @throws Exception
+	 */
+	public void reset(String name) throws Exception {
+		RefUpdate.Result result;
+		ObjectId id = db.resolve(name);
+		if (id == null)
+			throw new IOException("Not a revision: " + name);
+		RefUpdate ru = db.updateRef(Constants.HEAD, false);
+		ru.setNewObjectId(id);
+		result = ru.forceUpdate();
+		switch (result) {
+			case FAST_FORWARD:
+			case FORCED:
+			case NEW:
+			case NO_CHANGE:
+				break;
+			default:
+				throw new IOException(String.format(
+						"Checkout \"%s\" failed: %s", name, result));
+		}
+	}
+
+	/**
+	 * Cherry-pick a commit onto HEAD.
+	 * <p>
+	 * This differs from {@code git cherry-pick} in that it works in a bare
+	 * repository. As a result, any merge failure results in an exception, as
+	 * there is no way to recover.
+	 *
+	 * @param id
+	 *            commit-ish to cherry-pick.
+	 * @return newly created commit, or null if no work was done due to the
+	 *         resulting tree being identical.
+	 * @throws Exception
+	 */
+	public RevCommit cherryPick(AnyObjectId id) throws Exception {
+		RevCommit commit = pool.parseCommit(id);
+		pool.parseBody(commit);
+		if (commit.getParentCount() != 1)
+			throw new IOException(String.format(
+					"Expected 1 parent for %s, found: %s",
+					id.name(), Arrays.asList(commit.getParents())));
+		RevCommit parent = commit.getParent(0);
+		pool.parseHeaders(parent);
+
+		Ref headRef = db.exactRef(Constants.HEAD);
+		if (headRef == null)
+			throw new IOException("Missing HEAD");
+		RevCommit head = pool.parseCommit(headRef.getObjectId());
+
+		ThreeWayMerger merger = MergeStrategy.RECURSIVE.newMerger(db, true);
+		merger.setBase(parent.getTree());
+		if (merger.merge(head, commit)) {
+			if (AnyObjectId.equals(head.getTree(), merger.getResultTreeId()))
+				return null;
+			tick(1);
+			org.eclipse.jgit.lib.CommitBuilder b =
+					new org.eclipse.jgit.lib.CommitBuilder();
+			b.setParentId(head);
+			b.setTreeId(merger.getResultTreeId());
+			b.setAuthor(commit.getAuthorIdent());
+			b.setCommitter(new PersonIdent(defaultCommitter, getDate()));
+			b.setMessage(commit.getFullMessage());
+			ObjectId result;
+			try (ObjectInserter ins = inserter) {
+				result = ins.insert(b);
+				ins.flush();
+			}
+			update(Constants.HEAD, result);
+			return pool.parseCommit(result);
+		} else {
+			throw new IOException("Merge conflict");
+		}
+	}
+
 	/**
 	 * Update the dumb client server info files.
 	 *
@@ -518,7 +783,8 @@ public class TestRepository<R extends Repository> {
 	 * Ensure the body of the given object has been parsed.
 	 *
 	 * @param <T>
-	 *            type of object, e.g. {@link RevTag} or {@link RevCommit}.
+	 *            type of object, e.g. {@link org.eclipse.jgit.revwalk.RevTag}
+	 *            or {@link org.eclipse.jgit.revwalk.RevCommit}.
 	 * @param object
 	 *            reference to the (possibly unparsed) object to force body
 	 *            parsing of.
@@ -581,34 +847,35 @@ public class TestRepository<R extends Repository> {
 	 */
 	public void fsck(RevObject... tips) throws MissingObjectException,
 			IncorrectObjectTypeException, IOException {
-		ObjectWalk ow = new ObjectWalk(db);
-		if (tips.length != 0) {
-			for (RevObject o : tips)
-				ow.markStart(ow.parseAny(o));
-		} else {
-			for (Ref r : db.getAllRefs().values())
-				ow.markStart(ow.parseAny(r.getObjectId()));
-		}
+		try (ObjectWalk ow = new ObjectWalk(db)) {
+			if (tips.length != 0) {
+				for (RevObject o : tips)
+					ow.markStart(ow.parseAny(o));
+			} else {
+				for (Ref r : db.getAllRefs().values())
+					ow.markStart(ow.parseAny(r.getObjectId()));
+			}
 
-		ObjectChecker oc = new ObjectChecker();
-		for (;;) {
-			final RevCommit o = ow.next();
-			if (o == null)
-				break;
+			ObjectChecker oc = new ObjectChecker();
+			for (;;) {
+				final RevCommit o = ow.next();
+				if (o == null)
+					break;
 
-			final byte[] bin = db.open(o, o.getType()).getCachedBytes();
-			oc.checkCommit(bin);
-			assertHash(o, bin);
-		}
+				final byte[] bin = db.open(o, o.getType()).getCachedBytes();
+				oc.checkCommit(o, bin);
+				assertHash(o, bin);
+			}
 
-		for (;;) {
-			final RevObject o = ow.nextObject();
-			if (o == null)
-				break;
+			for (;;) {
+				final RevObject o = ow.nextObject();
+				if (o == null)
+					break;
 
-			final byte[] bin = db.open(o, o.getType()).getCachedBytes();
-			oc.check(o.getType(), bin);
-			assertHash(o, bin);
+				final byte[] bin = db.open(o, o.getType()).getCachedBytes();
+				oc.check(o, o.getType(), bin);
+				assertHash(o, bin);
+			}
 		}
 	}
 
@@ -636,35 +903,27 @@ public class TestRepository<R extends Repository> {
 			NullProgressMonitor m = NullProgressMonitor.INSTANCE;
 
 			final File pack, idx;
-			PackWriter pw = new PackWriter(db);
-			try {
-				Set<ObjectId> all = new HashSet<ObjectId>();
+			try (PackWriter pw = new PackWriter(db)) {
+				Set<ObjectId> all = new HashSet<>();
 				for (Ref r : db.getAllRefs().values())
 					all.add(r.getObjectId());
-				pw.preparePack(m, all, Collections.<ObjectId> emptySet());
+				pw.preparePack(m, all, PackWriter.NONE);
 
 				final ObjectId name = pw.computeName();
-				OutputStream out;
 
 				pack = nameFor(odb, name, ".pack");
-				out = new SafeBufferedOutputStream(new FileOutputStream(pack));
-				try {
+				try (OutputStream out =
+						new BufferedOutputStream(new FileOutputStream(pack))) {
 					pw.writePack(m, m, out);
-				} finally {
-					out.close();
 				}
 				pack.setReadOnly();
 
 				idx = nameFor(odb, name, ".idx");
-				out = new SafeBufferedOutputStream(new FileOutputStream(idx));
-				try {
+				try (OutputStream out =
+						new BufferedOutputStream(new FileOutputStream(idx))) {
 					pw.writeIndex(out);
-				} finally {
-					out.close();
 				}
 				idx.setReadOnly();
-			} finally {
-				pw.release();
 			}
 
 			odb.openPack(pack);
@@ -681,13 +940,13 @@ public class TestRepository<R extends Repository> {
 	}
 
 	private static File nameFor(ObjectDirectory odb, ObjectId name, String t) {
-		File packdir = new File(odb.getDirectory(), "pack");
+		File packdir = odb.getPackDirectory();
 		return new File(packdir, "pack-" + name.name() + t);
 	}
 
 	private void writeFile(final File p, final byte[] bin) throws IOException,
 			ObjectWritingException {
-		final LockFile lck = new LockFile(p, db.getFS());
+		final LockFile lck = new LockFile(p);
 		if (!lck.lock())
 			throw new ObjectWritingException("Can't write " + p);
 		try {
@@ -742,6 +1001,15 @@ public class TestRepository<R extends Repository> {
 		public RevCommit update(RevCommit to) throws Exception {
 			return TestRepository.this.update(ref, to);
 		}
+
+		/**
+		 * Delete this branch.
+		 * @throws Exception
+		 * @since 4.4
+		 */
+		public void delete() throws Exception {
+			TestRepository.this.delete(ref);
+		}
 	}
 
 	/** Helper to generate a commit. */
@@ -752,7 +1020,7 @@ public class TestRepository<R extends Repository> {
 
 		private ObjectId topLevelTree;
 
-		private final List<RevCommit> parents = new ArrayList<RevCommit>(2);
+		private final List<RevCommit> parents = new ArrayList<>(2);
 
 		private int tick = 1;
 
@@ -760,6 +1028,13 @@ public class TestRepository<R extends Repository> {
 
 		private RevCommit self;
 
+		private PersonIdent author;
+		private PersonIdent committer;
+
+		private String changeId;
+
+		private boolean updateCommitterTime;
+
 		CommitBuilder() {
 			branch = null;
 		}
@@ -767,10 +1042,9 @@ public class TestRepository<R extends Repository> {
 		CommitBuilder(BranchBuilder b) throws Exception {
 			branch = b;
 
-			Ref ref = db.getRef(branch.ref);
-			if (ref != null) {
+			Ref ref = db.exactRef(branch.ref);
+			if (ref != null && ref.getObjectId() != null)
 				parent(pool.parseCommit(ref.getObjectId()));
-			}
 		}
 
 		CommitBuilder(CommitBuilder prior) throws Exception {
@@ -796,6 +1070,10 @@ public class TestRepository<R extends Repository> {
 			return this;
 		}
 
+		public List<RevCommit> parents() {
+			return Collections.unmodifiableList(parents);
+		}
+
 		public CommitBuilder noParents() {
 			parents.clear();
 			return this;
@@ -846,11 +1124,51 @@ public class TestRepository<R extends Repository> {
 			return this;
 		}
 
+		public String message() {
+			return message;
+		}
+
 		public CommitBuilder tick(int secs) {
 			tick = secs;
 			return this;
 		}
 
+		public CommitBuilder ident(PersonIdent ident) {
+			author = ident;
+			committer = ident;
+			return this;
+		}
+
+		public CommitBuilder author(PersonIdent a) {
+			author = a;
+			return this;
+		}
+
+		public PersonIdent author() {
+			return author;
+		}
+
+		public CommitBuilder committer(PersonIdent c) {
+			committer = c;
+			return this;
+		}
+
+		public PersonIdent committer() {
+			return committer;
+		}
+
+		public CommitBuilder insertChangeId() {
+			changeId = "";
+			return this;
+		}
+
+		public CommitBuilder insertChangeId(String c) {
+			// Validate, but store as a string so we can use "" as a sentinel.
+			ObjectId.fromString(c);
+			changeId = c;
+			return this;
+		}
+
 		public RevCommit create() throws Exception {
 			if (self == null) {
 				TestRepository.this.tick(tick);
@@ -860,18 +1178,24 @@ public class TestRepository<R extends Repository> {
 				c = new org.eclipse.jgit.lib.CommitBuilder();
 				c.setParentIds(parents);
 				setAuthorAndCommitter(c);
-				c.setMessage(message);
+				if (author != null)
+					c.setAuthor(author);
+				if (committer != null) {
+					if (updateCommitterTime)
+						committer = new PersonIdent(committer, getDate());
+					c.setCommitter(committer);
+				}
 
 				ObjectId commitId;
-				try {
+				try (ObjectInserter ins = inserter) {
 					if (topLevelTree != null)
 						c.setTreeId(topLevelTree);
 					else
-						c.setTreeId(tree.writeTree(inserter));
-					commitId = inserter.insert(c);
-					inserter.flush();
-				} finally {
-					inserter.release();
+						c.setTreeId(tree.writeTree(ins));
+					insertChangeId(c);
+					c.setMessage(message);
+					commitId = ins.insert(c);
+					ins.flush();
 				}
 				self = pool.lookupCommit(commitId);
 
@@ -881,6 +1205,30 @@ public class TestRepository<R extends Repository> {
 			return self;
 		}
 
+		private void insertChangeId(org.eclipse.jgit.lib.CommitBuilder c) {
+			if (changeId == null)
+				return;
+			int idx = ChangeIdUtil.indexOfChangeId(message, "\n");
+			if (idx >= 0)
+				return;
+
+			ObjectId firstParentId = null;
+			if (!parents.isEmpty())
+				firstParentId = parents.get(0);
+
+			ObjectId cid;
+			if (changeId.equals(""))
+				cid = ChangeIdUtil.computeChangeId(c.getTreeId(), firstParentId,
+						c.getAuthor(), c.getCommitter(), message);
+			else
+				cid = ObjectId.fromString(changeId);
+			message = ChangeIdUtil.insertId(message, cid);
+			if (cid != null)
+				message = message.replaceAll("\nChange-Id: I" //$NON-NLS-1$
+						+ ObjectId.zeroId().getName() + "\n", "\nChange-Id: I" //$NON-NLS-1$ //$NON-NLS-2$
+						+ cid.getName() + "\n"); //$NON-NLS-1$
+		}
+
 		public CommitBuilder child() throws Exception {
 			return new CommitBuilder(this);
 		}
