@@ -68,13 +68,14 @@ class Config extends TextBuiltin {
 	@Option(name = "--file", aliases = { "-f" }, metaVar = "metaVar_file", usage = "usage_configFile")
 	private File configFile;
 
+	/** {@inheritDoc} */
 	@Override
 	protected void run() throws Exception {
 		if (list)
 			list();
 		else
 			throw new NotSupportedException(
-					"only --list option is currently supported");
+					"only --list option is currently supported"); //$NON-NLS-1$
 	}
 
 	private void list() throws IOException, ConfigInvalidException {
@@ -122,4 +123,4 @@ class Config extends TextBuiltin {
 			}
 		}
 	}
-}
\ No newline at end of file
+}
