@@ -44,8 +44,11 @@
 
 package org.eclipse.jgit.pgm;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
+
 import java.io.File;
 import java.io.IOException;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.lang.reflect.InvocationTargetException;
 import java.net.MalformedURLException;
@@ -53,22 +56,32 @@ import java.net.URL;
 import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Locale;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.ThreadFactory;
+import java.util.concurrent.TimeUnit;
 
 import org.eclipse.jgit.awtui.AwtAuthenticator;
 import org.eclipse.jgit.awtui.AwtCredentialsProvider;
 import org.eclipse.jgit.errors.TransportException;
+import org.eclipse.jgit.lfs.BuiltinLFS;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryBuilder;
 import org.eclipse.jgit.pgm.internal.CLIText;
 import org.eclipse.jgit.pgm.opt.CmdLineParser;
 import org.eclipse.jgit.pgm.opt.SubcommandHandler;
+import org.eclipse.jgit.transport.HttpTransport;
+import org.eclipse.jgit.transport.http.apache.HttpClientConnectionFactory;
 import org.eclipse.jgit.util.CachedAuthenticator;
 import org.kohsuke.args4j.Argument;
 import org.kohsuke.args4j.CmdLineException;
-import org.kohsuke.args4j.ExampleMode;
 import org.kohsuke.args4j.Option;
+import org.kohsuke.args4j.OptionHandlerFilter;
 
-/** Command line entry point. */
+/**
+ * Command line entry point.
+ */
 public class Main {
 	@Option(name = "--help", usage = "usage_displayThisHelpText", aliases = { "-h" })
 	private boolean help;
@@ -86,15 +99,42 @@ public class Main {
 	private TextBuiltin subcommand;
 
 	@Argument(index = 1, metaVar = "metaVar_arg")
-	private List<String> arguments = new ArrayList<String>();
+	private List<String> arguments = new ArrayList<>();
+
+	PrintWriter writer;
+
+	private ExecutorService gcExecutor;
+
+	/**
+	 * <p>Constructor for Main.</p>
+	 */
+	public Main() {
+		HttpTransport.setConnectionFactory(new HttpClientConnectionFactory());
+		BuiltinLFS.register();
+		gcExecutor = Executors.newSingleThreadExecutor(new ThreadFactory() {
+			private final ThreadFactory baseFactory = Executors
+					.defaultThreadFactory();
+
+			@Override
+			public Thread newThread(Runnable taskBody) {
+				Thread thr = baseFactory.newThread(taskBody);
+				thr.setName("JGit-autoGc"); //$NON-NLS-1$
+				return thr;
+			}
+		});
+	}
 
 	/**
 	 * Execute the command line.
 	 *
 	 * @param argv
 	 *            arguments.
+	 * @throws java.lang.Exception
 	 */
-	public static void main(final String[] argv) {
+	public static void main(final String[] argv) throws Exception {
+		// make sure built-in filters are registered
+		BuiltinLFS.register();
+
 		new Main().run(argv);
 	}
 
@@ -113,8 +153,10 @@ public class Main {
 	 *
 	 * @param argv
 	 *            arguments.
+	 * @throws java.lang.Exception
 	 */
-	protected void run(final String[] argv) {
+	protected void run(final String[] argv) throws Exception {
+		writer = createErrorWriter();
 		try {
 			if (!installConsole()) {
 				AwtAuthenticator.install();
@@ -123,12 +165,14 @@ public class Main {
 			configureHttpProxy();
 			execute(argv);
 		} catch (Die err) {
-			if (err.isAborted())
-				System.exit(1);
-			System.err.println(MessageFormat.format(CLIText.get().fatalError, err.getMessage()));
-			if (showStackTrace)
-				err.printStackTrace();
-			System.exit(128);
+			if (err.isAborted()) {
+				exit(1, err);
+			}
+			writer.println(CLIText.fatalError(err.getMessage()));
+			if (showStackTrace) {
+				err.printStackTrace(writer);
+			}
+			exit(128, err);
 		} catch (Exception err) {
 			// Try to detect errno == EPIPE and exit normally if that happens
 			// There may be issues with operating system versions and locale,
@@ -136,52 +180,63 @@ public class Main {
 			// under other circumstances.
 			if (err.getClass() == IOException.class) {
 				// Linux, OS X
-				if (err.getMessage().equals("Broken pipe")) //$NON-NLS-1$
-					System.exit(0);
+				if (err.getMessage().equals("Broken pipe")) { //$NON-NLS-1$
+					exit(0, err);
+				}
 				// Windows
-				if (err.getMessage().equals("The pipe is being closed")) //$NON-NLS-1$
-					System.exit(0);
+				if (err.getMessage().equals("The pipe is being closed")) { //$NON-NLS-1$
+					exit(0, err);
+				}
 			}
 			if (!showStackTrace && err.getCause() != null
-					&& err instanceof TransportException)
-				System.err.println(MessageFormat.format(CLIText.get().fatalError, err.getCause().getMessage()));
+					&& err instanceof TransportException) {
+				writer.println(CLIText.fatalError(err.getCause().getMessage()));
+			}
 
 			if (err.getClass().getName().startsWith("org.eclipse.jgit.errors.")) { //$NON-NLS-1$
-				System.err.println(MessageFormat.format(CLIText.get().fatalError, err.getMessage()));
-				if (showStackTrace)
+				writer.println(CLIText.fatalError(err.getMessage()));
+				if (showStackTrace) {
 					err.printStackTrace();
-				System.exit(128);
+				}
+				exit(128, err);
 			}
 			err.printStackTrace();
-			System.exit(1);
+			exit(1, err);
 		}
 		if (System.out.checkError()) {
-			System.err.println(CLIText.get().unknownIoErrorStdout);
-			System.exit(1);
+			writer.println(CLIText.get().unknownIoErrorStdout);
+			exit(1, null);
 		}
-		if (System.err.checkError()) {
+		if (writer.checkError()) {
 			// No idea how to present an error here, most likely disk full or
 			// broken pipe
-			System.exit(1);
+			exit(1, null);
 		}
+		gcExecutor.shutdown();
+		gcExecutor.awaitTermination(10, TimeUnit.MINUTES);
+	}
+
+	PrintWriter createErrorWriter() {
+		return new PrintWriter(new OutputStreamWriter(System.err, UTF_8));
 	}
 
 	private void execute(final String[] argv) throws Exception {
-		final CmdLineParser clp = new CmdLineParser(this);
-		PrintWriter writer = new PrintWriter(System.err);
+		final CmdLineParser clp = new SubcommandLineParser(this);
+
 		try {
 			clp.parseArgument(argv);
 		} catch (CmdLineException err) {
 			if (argv.length > 0 && !help && !version) {
-				writer.println(MessageFormat.format(CLIText.get().fatalError, err.getMessage()));
+				writer.println(CLIText.fatalError(err.getMessage()));
 				writer.flush();
-				System.exit(1);
+				exit(1, err);
 			}
 		}
 
 		if (argv.length == 0 || help) {
-			final String ex = clp.printExample(ExampleMode.ALL, CLIText.get().resourceBundle());
-			writer.println("jgit" + ex + " command [ARG ...]"); //$NON-NLS-1$
+			final String ex = clp.printExample(OptionHandlerFilter.ALL,
+					CLIText.get().resourceBundle());
+			writer.println("jgit" + ex + " command [ARG ...]"); //$NON-NLS-1$ //$NON-NLS-2$
 			if (help) {
 				writer.println();
 				clp.printUsage(writer, CLIText.get().resourceBundle());
@@ -191,44 +246,65 @@ public class Main {
 				writer.println(CLIText.get().mostCommonlyUsedCommandsAre);
 				final CommandRef[] common = CommandCatalog.common();
 				int width = 0;
-				for (final CommandRef c : common)
+				for (final CommandRef c : common) {
 					width = Math.max(width, c.getName().length());
+				}
 				width += 2;
 
 				for (final CommandRef c : common) {
 					writer.print(' ');
 					writer.print(c.getName());
-					for (int i = c.getName().length(); i < width; i++)
+					for (int i = c.getName().length(); i < width; i++) {
 						writer.print(' ');
+					}
 					writer.print(CLIText.get().resourceBundle().getString(c.getUsage()));
 					writer.println();
 				}
 				writer.println();
 			}
 			writer.flush();
-			System.exit(1);
+			exit(1, null);
 		}
 
 		if (version) {
-			String cmdId = Version.class.getSimpleName().toLowerCase();
+			String cmdId = Version.class.getSimpleName()
+					.toLowerCase(Locale.ROOT);
 			subcommand = CommandCatalog.get(cmdId).create();
 		}
 
 		final TextBuiltin cmd = subcommand;
-		if (cmd.requiresRepository())
-			cmd.init(openGitDir(gitdir), null);
-		else
-			cmd.init(null, gitdir);
+		init(cmd);
 		try {
 			cmd.execute(arguments.toArray(new String[arguments.size()]));
 		} finally {
-			if (cmd.outw != null)
+			if (cmd.outw != null) {
 				cmd.outw.flush();
-			if (cmd.errw != null)
+			}
+			if (cmd.errw != null) {
 				cmd.errw.flush();
+			}
 		}
 	}
 
+	void init(final TextBuiltin cmd) throws IOException {
+		if (cmd.requiresRepository()) {
+			cmd.init(openGitDir(gitdir), null);
+		} else {
+			cmd.init(null, gitdir);
+		}
+	}
+
+	/**
+	 * @param status
+	 * @param t
+	 *            can be {@code null}
+	 * @throws Exception
+	 */
+	void exit(int status, Exception t) throws Exception {
+		writer.flush();
+		System.exit(status);
+	}
+
 	/**
 	 * Evaluate the {@code --git-dir} option and open the repository.
 	 *
@@ -236,7 +312,7 @@ public class Main {
 	 *            the {@code --git-dir} option given on the command line. May be
 	 *            null if it was not supplied.
 	 * @return the repository to operate on.
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             the repository cannot be opened.
 	 */
 	protected Repository openGitDir(String aGitdir) throws IOException {
@@ -278,7 +354,7 @@ public class Main {
 			throws IllegalAccessException, InvocationTargetException,
 			NoSuchMethodException, ClassNotFoundException {
 		try {
-		Class.forName(name).getMethod("install").invoke(null); //$NON-NLS-1$
+			Class.forName(name).getMethod("install").invoke(null); //$NON-NLS-1$
 		} catch (InvocationTargetException e) {
 			if (e.getCause() instanceof RuntimeException)
 				throw (RuntimeException) e.getCause();
@@ -291,39 +367,71 @@ public class Main {
 	/**
 	 * Configure the JRE's standard HTTP based on <code>http_proxy</code>.
 	 * <p>
-	 * The popular libcurl library honors the <code>http_proxy</code>
-	 * environment variable as a means of specifying an HTTP proxy for requests
-	 * made behind a firewall. This is not natively recognized by the JRE, so
-	 * this method can be used by command line utilities to configure the JRE
-	 * before the first request is sent.
+	 * The popular libcurl library honors the <code>http_proxy</code>,
+	 * <code>https_proxy</code> environment variables as a means of specifying
+	 * an HTTP/S proxy for requests made behind a firewall. This is not natively
+	 * recognized by the JRE, so this method can be used by command line
+	 * utilities to configure the JRE before the first request is sent. The
+	 * information found in the environment variables is copied to the
+	 * associated system properties. This is not done when the system properties
+	 * are already set. The default way of telling java programs about proxies
+	 * (the system properties) takes precedence over environment variables.
 	 *
 	 * @throws MalformedURLException
-	 *             the value in <code>http_proxy</code> is unsupportable.
+	 *             the value in <code>http_proxy</code> or
+	 *             <code>https_proxy</code> is unsupportable.
 	 */
-	private static void configureHttpProxy() throws MalformedURLException {
-		final String s = System.getenv("http_proxy"); //$NON-NLS-1$
-		if (s == null || s.equals("")) //$NON-NLS-1$
-			return;
+	static void configureHttpProxy() throws MalformedURLException {
+		for (String protocol : new String[] { "http", "https" }) { //$NON-NLS-1$ //$NON-NLS-2$
+			if (System.getProperty(protocol + ".proxyHost") != null) { //$NON-NLS-1$
+				continue;
+			}
+			String s = System.getenv(protocol + "_proxy"); //$NON-NLS-1$
+			if (s == null && protocol.equals("https")) { //$NON-NLS-1$
+				s = System.getenv("HTTPS_PROXY"); //$NON-NLS-1$
+			}
+			if (s == null || s.equals("")) { //$NON-NLS-1$
+				continue;
+			}
 
-		final URL u = new URL((s.indexOf("://") == -1) ? "http://" + s : s); //$NON-NLS-1$ //$NON-NLS-2$
-		if (!"http".equals(u.getProtocol())) //$NON-NLS-1$
-			throw new MalformedURLException(MessageFormat.format(CLIText.get().invalidHttpProxyOnlyHttpSupported, s));
+			final URL u = new URL(
+					(s.indexOf("://") == -1) ? protocol + "://" + s : s); //$NON-NLS-1$ //$NON-NLS-2$
+			if (!u.getProtocol().startsWith("http")) //$NON-NLS-1$
+				throw new MalformedURLException(MessageFormat.format(
+						CLIText.get().invalidHttpProxyOnlyHttpSupported, s));
 
-		final String proxyHost = u.getHost();
-		final int proxyPort = u.getPort();
+			final String proxyHost = u.getHost();
+			final int proxyPort = u.getPort();
 
-		System.setProperty("http.proxyHost", proxyHost); //$NON-NLS-1$
-		if (proxyPort > 0)
-			System.setProperty("http.proxyPort", String.valueOf(proxyPort)); //$NON-NLS-1$
+			System.setProperty(protocol + ".proxyHost", proxyHost); //$NON-NLS-1$
+			if (proxyPort > 0)
+				System.setProperty(protocol + ".proxyPort", //$NON-NLS-1$
+						String.valueOf(proxyPort));
 
-		final String userpass = u.getUserInfo();
-		if (userpass != null && userpass.contains(":")) { //$NON-NLS-1$
-			final int c = userpass.indexOf(':');
-			final String user = userpass.substring(0, c);
-			final String pass = userpass.substring(c + 1);
-			CachedAuthenticator
-					.add(new CachedAuthenticator.CachedAuthentication(
-							proxyHost, proxyPort, user, pass));
+			final String userpass = u.getUserInfo();
+			if (userpass != null && userpass.contains(":")) { //$NON-NLS-1$
+				final int c = userpass.indexOf(':');
+				final String user = userpass.substring(0, c);
+				final String pass = userpass.substring(c + 1);
+				CachedAuthenticator.add(
+						new CachedAuthenticator.CachedAuthentication(proxyHost,
+								proxyPort, user, pass));
+			}
+		}
+	}
+
+	/**
+	 * Parser for subcommands which doesn't stop parsing on help options and so
+	 * proceeds all specified options
+	 */
+	static class SubcommandLineParser extends CmdLineParser {
+		public SubcommandLineParser(Object bean) {
+			super(bean);
+		}
+
+		@Override
+		protected boolean containsHelp(String... args) {
+			return false;
 		}
 	}
 }
