@@ -49,8 +49,8 @@ import java.util.Map;
 
 import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.api.MergeCommand;
-import org.eclipse.jgit.api.MergeResult;
 import org.eclipse.jgit.api.MergeCommand.FastForwardMode;
+import org.eclipse.jgit.api.MergeResult;
 import org.eclipse.jgit.api.errors.CheckoutConflictException;
 import org.eclipse.jgit.lib.AnyObjectId;
 import org.eclipse.jgit.lib.Constants;
@@ -101,6 +101,7 @@ class Merge extends TextBuiltin {
 	@Option(name = "-m", usage = "usage_message")
 	private String message;
 
+	/** {@inheritDoc} */
 	@Override
 	protected void run() throws Exception {
 		if (squash && ff == FastForwardMode.NO_FF)
@@ -114,29 +115,30 @@ class Merge extends TextBuiltin {
 		}
 
 		// determine the other revision we want to merge with HEAD
-		final Ref srcRef = db.getRef(ref);
+		final Ref srcRef = db.findRef(ref);
 		final ObjectId src = db.resolve(ref + "^{commit}"); //$NON-NLS-1$
 		if (src == null)
 			throw die(MessageFormat.format(
 					CLIText.get().refDoesNotExistOrNoCommit, ref));
 
-		Ref oldHead = db.getRef(Constants.HEAD);
-		Git git = new Git(db);
-		MergeCommand mergeCmd = git.merge().setStrategy(mergeStrategy)
-				.setSquash(squash).setFastForward(ff).setCommit(!noCommit);
-		if (srcRef != null)
-			mergeCmd.include(srcRef);
-		else
-			mergeCmd.include(src);
-
-		if (message != null)
-			mergeCmd.setMessage(message);
-
+		Ref oldHead = getOldHead();
 		MergeResult result;
-		try {
-			result = mergeCmd.call();
-		} catch (CheckoutConflictException e) {
-			result = new MergeResult(e.getConflictingPaths()); // CHECKOUT_CONFLICT
+		try (Git git = new Git(db)) {
+			MergeCommand mergeCmd = git.merge().setStrategy(mergeStrategy)
+					.setSquash(squash).setFastForward(ff).setCommit(!noCommit);
+			if (srcRef != null)
+				mergeCmd.include(srcRef);
+			else
+				mergeCmd.include(src);
+
+			if (message != null)
+				mergeCmd.setMessage(message);
+
+			try {
+				result = mergeCmd.call();
+			} catch (CheckoutConflictException e) {
+				result = new MergeResult(e.getConflictingPaths()); // CHECKOUT_CONFLICT
+			}
 		}
 
 		switch (result.getMergeStatus()) {
@@ -147,9 +149,12 @@ class Merge extends TextBuiltin {
 			break;
 		case FAST_FORWARD:
 			ObjectId oldHeadId = oldHead.getObjectId();
-			outw.println(MessageFormat.format(CLIText.get().updating, oldHeadId
-					.abbreviate(7).name(), result.getNewHead().abbreviate(7)
-					.name()));
+			if (oldHeadId != null) {
+				String oldId = oldHeadId.abbreviate(7).name();
+				String newId = result.getNewHead().abbreviate(7).name();
+				outw.println(MessageFormat.format(CLIText.get().updating, oldId,
+						newId));
+			}
 			outw.println(result.getMergeStatus().toString());
 			break;
 		case CHECKOUT_CONFLICT:
@@ -204,14 +209,23 @@ class Merge extends TextBuiltin {
 		}
 	}
 
+	private Ref getOldHead() throws IOException {
+		Ref oldHead = db.exactRef(Constants.HEAD);
+		if (oldHead == null) {
+			throw die(CLIText.get().onBranchToBeBorn);
+		}
+		return oldHead;
+	}
+
 	private boolean isMergedInto(Ref oldHead, AnyObjectId src)
 			throws IOException {
-		RevWalk revWalk = new RevWalk(db);
-		ObjectId oldHeadObjectId = oldHead.getPeeledObjectId();
-		if (oldHeadObjectId == null)
-			oldHeadObjectId = oldHead.getObjectId();
-		RevCommit oldHeadCommit = revWalk.lookupCommit(oldHeadObjectId);
-		RevCommit srcCommit = revWalk.lookupCommit(src);
-		return revWalk.isMergedInto(oldHeadCommit, srcCommit);
+		try (RevWalk revWalk = new RevWalk(db)) {
+			ObjectId oldHeadObjectId = oldHead.getPeeledObjectId();
+			if (oldHeadObjectId == null)
+				oldHeadObjectId = oldHead.getObjectId();
+			RevCommit oldHeadCommit = revWalk.lookupCommit(oldHeadObjectId);
+			RevCommit srcCommit = revWalk.lookupCommit(src);
+			return revWalk.isMergedInto(oldHeadCommit, srcCommit);
+		}
 	}
 }
