@@ -59,6 +59,7 @@ class ShowCommands extends TextBuiltin {
 	@Option(name = "--pretty", metaVar = "metaVar_commandDetail", usage = "usage_alterTheDetailShown")
 	private Format pretty = Format.USAGE;
 
+	/** {@inheritDoc} */
 	@Override
 	protected void run() throws Exception {
 		final CommandRef[] list = CommandCatalog.all();
@@ -85,6 +86,7 @@ class ShowCommands extends TextBuiltin {
 	static enum Format {
 		/** */
 		USAGE {
+			@Override
 			void print(ThrowingPrintWriter err, final CommandRef c) throws IOException {
 				String usage = c.getUsage();
 				if (usage != null && usage.length() > 0)
@@ -94,6 +96,7 @@ class ShowCommands extends TextBuiltin {
 
 		/** */
 		CLASSES {
+			@Override
 			void print(ThrowingPrintWriter err, final CommandRef c) throws IOException {
 				err.print(c.getImplementationClassName());
 			}
@@ -101,6 +104,7 @@ class ShowCommands extends TextBuiltin {
 
 		/** */
 		URLS {
+			@Override
 			void print(ThrowingPrintWriter err, final CommandRef c) throws IOException {
 				final ClassLoader ldr = c.getImplementationClassLoader();
 
