@@ -42,6 +42,7 @@
  */
 package org.eclipse.jgit.api;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
@@ -61,6 +62,7 @@ import org.eclipse.jgit.junit.RepositoryTestCase;
 import org.eclipse.jgit.lib.ConfigConstants;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.ObjectId;
+import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.lib.RepositoryState;
 import org.eclipse.jgit.lib.StoredConfig;
@@ -147,6 +149,57 @@ public class PullCommandWithRebaseTest extends RepositoryTestCase {
 
 	}
 
+	@Test
+	public void testPullFastForwardDetachedHead() throws Exception {
+		Repository repository = source.getRepository();
+		writeToFile(sourceFile, "2nd commit");
+		source.add().addFilepattern("SomeFile.txt").call();
+		source.commit().setMessage("2nd commit").call();
+
+		try (RevWalk revWalk = new RevWalk(repository)) {
+			// git checkout HEAD^
+			String initialBranch = repository.getBranch();
+			Ref initialRef = repository.findRef(Constants.HEAD);
+			RevCommit initialCommit = revWalk
+					.parseCommit(initialRef.getObjectId());
+			assertEquals("this test need linear history", 1,
+					initialCommit.getParentCount());
+			source.checkout().setName(initialCommit.getParent(0).getName())
+					.call();
+			assertFalse("expected detached HEAD",
+					repository.getFullBranch().startsWith(Constants.R_HEADS));
+
+			// change and commit another file
+			File otherFile = new File(sourceFile.getParentFile(),
+					System.currentTimeMillis() + ".tst");
+			writeToFile(otherFile, "other 2nd commit");
+			source.add().addFilepattern(otherFile.getName()).call();
+			RevCommit newCommit = source.commit().setMessage("other 2nd commit")
+					.call();
+
+			// git pull --rebase initialBranch
+			source.pull().setRebase(true).setRemote(".")
+					.setRemoteBranchName(initialBranch)
+					.call();
+
+			assertEquals(RepositoryState.SAFE,
+					source.getRepository().getRepositoryState());
+			Ref head = source.getRepository().findRef(Constants.HEAD);
+			RevCommit headCommit = revWalk.parseCommit(head.getObjectId());
+
+			// HEAD^ == initialCommit, no merge commit
+			assertEquals(1, headCommit.getParentCount());
+			assertEquals(initialCommit, headCommit.getParent(0));
+
+			// both contributions for both commits are available
+			assertFileContentsEqual(sourceFile, "2nd commit");
+			assertFileContentsEqual(otherFile, "other 2nd commit");
+			// HEAD has same message as rebased commit
+			assertEquals(newCommit.getShortMessage(),
+					headCommit.getShortMessage());
+		}
+	}
+
 	@Test
 	public void testPullConflict() throws Exception {
 		PullResult res = target.pull().call();
@@ -180,7 +233,7 @@ public class PullCommandWithRebaseTest extends RepositoryTestCase {
 				+ remoteUri
 				+ "\nSource change\n=======\nTarget change\n>>>>>>> 42453fd Target change in local\n";
 		assertFileContentsEqual(targetFile, result);
-		assertEquals(RepositoryState.REBASING_INTERACTIVE, target
+		assertEquals(RepositoryState.REBASING_MERGE, target
 				.getRepository().getRepositoryState());
 	}
 
@@ -225,7 +278,7 @@ public class PullCommandWithRebaseTest extends RepositoryTestCase {
 		String result = "<<<<<<< Upstream, based on branch 'master' of local repository\n"
 				+ "Master change\n=======\nSlave change\n>>>>>>> 4049c9e Source change in based on master\n";
 		assertFileContentsEqual(targetFile, result);
-		assertEquals(RepositoryState.REBASING_INTERACTIVE, target
+		assertEquals(RepositoryState.REBASING_MERGE, target
 				.getRepository().getRepositoryState());
 	}
 
@@ -273,26 +326,27 @@ public class PullCommandWithRebaseTest extends RepositoryTestCase {
 
 		// Get the HEAD and HEAD~1 commits
 		Repository targetRepo = target.getRepository();
-		RevWalk revWalk = new RevWalk(targetRepo);
-		ObjectId headId = targetRepo.resolve(Constants.HEAD);
-		RevCommit root = revWalk.parseCommit(headId);
-		revWalk.markStart(root);
-		// HEAD
-		RevCommit head = revWalk.next();
-		// HEAD~1
-		RevCommit beforeHead = revWalk.next();
+		try (RevWalk revWalk = new RevWalk(targetRepo)) {
+			ObjectId headId = targetRepo.resolve(Constants.HEAD);
+			RevCommit root = revWalk.parseCommit(headId);
+			revWalk.markStart(root);
+			// HEAD
+			RevCommit head = revWalk.next();
+			// HEAD~1
+			RevCommit beforeHead = revWalk.next();
 
-		// verify the commit message on the HEAD commit
-		assertEquals(TARGET_COMMIT_MESSAGE, head.getFullMessage());
-		// verify the commit just before HEAD
-		assertEquals(SOURCE_COMMIT_MESSAGE, beforeHead.getFullMessage());
+			// verify the commit message on the HEAD commit
+			assertEquals(TARGET_COMMIT_MESSAGE, head.getFullMessage());
+			// verify the commit just before HEAD
+			assertEquals(SOURCE_COMMIT_MESSAGE, beforeHead.getFullMessage());
 
-		// verify file states
-		assertFileContentsEqual(sourceFile, SOURCE_FILE_CONTENTS);
-		assertFileContentsEqual(newFile, NEW_FILE_CONTENTS);
-		// verify repository state
-		assertEquals(RepositoryState.SAFE, target
-			.getRepository().getRepositoryState());
+			// verify file states
+			assertFileContentsEqual(sourceFile, SOURCE_FILE_CONTENTS);
+			assertFileContentsEqual(newFile, NEW_FILE_CONTENTS);
+			// verify repository state
+			assertEquals(RepositoryState.SAFE, target
+				.getRepository().getRepositoryState());
+		}
 	}
 
 	@Override
@@ -341,34 +395,23 @@ public class PullCommandWithRebaseTest extends RepositoryTestCase {
 
 	private static void writeToFile(File actFile, String string)
 			throws IOException {
-		FileOutputStream fos = null;
-		try {
-			fos = new FileOutputStream(actFile);
-			fos.write(string.getBytes("UTF-8"));
-			fos.close();
-		} finally {
-			if (fos != null)
-				fos.close();
+		try (FileOutputStream fos = new FileOutputStream(actFile)) {
+			fos.write(string.getBytes(UTF_8));
 		}
 	}
 
 	private static void assertFileContentsEqual(File actFile, String string)
 			throws IOException {
 		ByteArrayOutputStream bos = new ByteArrayOutputStream();
-		FileInputStream fis = null;
 		byte[] buffer = new byte[100];
-		try {
-			fis = new FileInputStream(actFile);
+		try (FileInputStream fis = new FileInputStream(actFile)) {
 			int read = fis.read(buffer);
 			while (read > 0) {
 				bos.write(buffer, 0, read);
 				read = fis.read(buffer);
 			}
-			String content = new String(bos.toByteArray(), "UTF-8");
+			String content = new String(bos.toByteArray(), UTF_8);
 			assertEquals(string, content);
-		} finally {
-			if (fis != null)
-				fis.close();
 		}
 	}
 }
