@@ -65,6 +65,7 @@ import org.eclipse.jgit.junit.RepositoryTestCase;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.FileMode;
 import org.eclipse.jgit.lib.ObjectId;
+import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.treewalk.TreeWalk;
@@ -139,8 +140,8 @@ public class ResetCommandTest extends RepositoryTestCase {
 			AmbiguousObjectException, IOException, GitAPIException {
 		setupRepository();
 		ObjectId prevHead = db.resolve(Constants.HEAD);
-		git.reset().setMode(ResetType.HARD).setRef(initialCommit.getName())
-				.call();
+		assertSameAsHead(git.reset().setMode(ResetType.HARD)
+				.setRef(initialCommit.getName()).call());
 		// check if HEAD points to initial commit now
 		ObjectId head = db.resolve(Constants.HEAD);
 		assertEquals(initialCommit, head);
@@ -155,6 +156,56 @@ public class ResetCommandTest extends RepositoryTestCase {
 		assertEquals(prevHead, db.readOrigHead());
 	}
 
+	@Test
+	public void testHardResetReflogDisabled() throws Exception {
+		setupRepository();
+		ObjectId prevHead = db.resolve(Constants.HEAD);
+		ResetCommand reset = git.reset();
+		assertSameAsHead(reset.setMode(ResetType.HARD)
+				.setRef(initialCommit.getName()).disableRefLog(true).call());
+		assertTrue("reflog should be disabled", reset.isReflogDisabled());
+		// check if HEAD points to initial commit now
+		ObjectId head = db.resolve(Constants.HEAD);
+		assertEquals(initialCommit, head);
+		// check if files were removed
+		assertFalse(indexFile.exists());
+		assertTrue(untrackedFile.exists());
+		// fileInIndex must no longer be in HEAD and in the index
+		String fileInIndexPath = indexFile.getAbsolutePath();
+		assertFalse(inHead(fileInIndexPath));
+		assertFalse(inIndex(indexFile.getName()));
+		assertReflogDisabled(head);
+		assertEquals(prevHead, db.readOrigHead());
+	}
+
+	@Test
+	public void testHardResetWithConflicts_DoOverWriteUntrackedFile()
+			throws JGitInternalException,
+			AmbiguousObjectException, IOException, GitAPIException {
+		setupRepository();
+		git.rm().setCached(true).addFilepattern("a.txt").call();
+		assertTrue(new File(db.getWorkTree(), "a.txt").exists());
+		git.reset().setMode(ResetType.HARD).setRef(Constants.HEAD)
+				.call();
+		assertTrue(new File(db.getWorkTree(), "a.txt").exists());
+		assertEquals("content", read(new File(db.getWorkTree(), "a.txt")));
+	}
+
+	@Test
+	public void testHardResetWithConflicts_DoDeleteFileFolderConflicts()
+			throws JGitInternalException,
+			AmbiguousObjectException, IOException, GitAPIException {
+		setupRepository();
+		writeTrashFile("d/c.txt", "x");
+		git.add().addFilepattern("d/c.txt").call();
+		FileUtils.delete(new File(db.getWorkTree(), "d"), FileUtils.RECURSIVE);
+		writeTrashFile("d", "y");
+
+		git.reset().setMode(ResetType.HARD).setRef(Constants.HEAD)
+				.call();
+		assertFalse(new File(db.getWorkTree(), "d").exists());
+	}
+
 	@Test
 	public void testResetToNonexistingHEAD() throws JGitInternalException,
 			AmbiguousObjectException, IOException, GitAPIException {
@@ -176,8 +227,8 @@ public class ResetCommandTest extends RepositoryTestCase {
 			AmbiguousObjectException, IOException, GitAPIException {
 		setupRepository();
 		ObjectId prevHead = db.resolve(Constants.HEAD);
-		git.reset().setMode(ResetType.SOFT).setRef(initialCommit.getName())
-				.call();
+		assertSameAsHead(git.reset().setMode(ResetType.SOFT)
+				.setRef(initialCommit.getName()).call());
 		// check if HEAD points to initial commit now
 		ObjectId head = db.resolve(Constants.HEAD);
 		assertEquals(initialCommit, head);
@@ -197,8 +248,8 @@ public class ResetCommandTest extends RepositoryTestCase {
 			AmbiguousObjectException, IOException, GitAPIException {
 		setupRepository();
 		ObjectId prevHead = db.resolve(Constants.HEAD);
-		git.reset().setMode(ResetType.MIXED).setRef(initialCommit.getName())
-				.call();
+		assertSameAsHead(git.reset().setMode(ResetType.MIXED)
+				.setRef(initialCommit.getName()).call());
 		// check if HEAD points to initial commit now
 		ObjectId head = db.resolve(Constants.HEAD);
 		assertEquals(initialCommit, head);
@@ -241,7 +292,8 @@ public class ResetCommandTest extends RepositoryTestCase {
 		assertTrue(bEntry.getLength() > 0);
 		assertTrue(bEntry.getLastModified() > 0);
 
-		git.reset().setMode(ResetType.MIXED).setRef(commit2.getName()).call();
+		assertSameAsHead(git.reset().setMode(ResetType.MIXED)
+				.setRef(commit2.getName()).call());
 
 		cache = db.readDirCache();
 
@@ -280,7 +332,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 				+ "[a.txt, mode:100644, stage:3]",
 				indexState(0));
 
-		git.reset().setMode(ResetType.MIXED).call();
+		assertSameAsHead(git.reset().setMode(ResetType.MIXED).call());
 
 		assertEquals("[a.txt, mode:100644]" + "[b.txt, mode:100644]",
 				indexState(0));
@@ -298,8 +350,8 @@ public class ResetCommandTest extends RepositoryTestCase {
 
 		// 'a.txt' has already been modified in setupRepository
 		// 'notAddedToIndex.txt' has been added to repository
-		git.reset().addPath(indexFile.getName())
-				.addPath(untrackedFile.getName()).call();
+		assertSameAsHead(git.reset().addPath(indexFile.getName())
+				.addPath(untrackedFile.getName()).call());
 
 		DirCacheEntry postReset = DirCache.read(db.getIndexFile(), db.getFS())
 				.getEntry(indexFile.getName());
@@ -329,7 +381,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 		git.add().addFilepattern(untrackedFile.getName()).call();
 
 		// 'dir/b.txt' has already been modified in setupRepository
-		git.reset().addPath("dir").call();
+		assertSameAsHead(git.reset().addPath("dir").call());
 
 		DirCacheEntry postReset = DirCache.read(db.getIndexFile(), db.getFS())
 				.getEntry("dir/b.txt");
@@ -358,9 +410,9 @@ public class ResetCommandTest extends RepositoryTestCase {
 		// 'a.txt' has already been modified in setupRepository
 		// 'notAddedToIndex.txt' has been added to repository
 		// reset to the inital commit
-		git.reset().setRef(initialCommit.getName())
-				.addPath(indexFile.getName())
-				.addPath(untrackedFile.getName()).call();
+		assertSameAsHead(git.reset().setRef(initialCommit.getName())
+				.addPath(indexFile.getName()).addPath(untrackedFile.getName())
+				.call());
 
 		// check that HEAD hasn't moved
 		ObjectId head = db.resolve(Constants.HEAD);
@@ -397,7 +449,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 				+ "[b.txt, mode:100644]",
 				indexState(0));
 
-		git.reset().addPath(file).call();
+		assertSameAsHead(git.reset().addPath(file).call());
 
 		assertEquals("[a.txt, mode:100644]" + "[b.txt, mode:100644]",
 				indexState(0));
@@ -409,7 +461,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 		writeTrashFile("a.txt", "content");
 		git.add().addFilepattern("a.txt").call();
 		// Should assume an empty tree, like in C Git 1.8.2
-		git.reset().addPath("a.txt").call();
+		assertSameAsHead(git.reset().addPath("a.txt").call());
 
 		DirCache cache = db.readDirCache();
 		DirCacheEntry aEntry = cache.getEntry("a.txt");
@@ -421,7 +473,8 @@ public class ResetCommandTest extends RepositoryTestCase {
 		git = new Git(db);
 		writeTrashFile("a.txt", "content");
 		git.add().addFilepattern("a.txt").call();
-		git.reset().setRef("doesnotexist").addPath("a.txt").call();
+		assertSameAsHead(
+				git.reset().setRef("doesnotexist").addPath("a.txt").call());
 	}
 
 	@Test
@@ -431,7 +484,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 		git.add().addFilepattern("a.txt").call();
 		writeTrashFile("a.txt", "modified");
 		// should use default mode MIXED
-		git.reset().call();
+		assertSameAsHead(git.reset().call());
 
 		DirCache cache = db.readDirCache();
 		DirCacheEntry aEntry = cache.getEntry("a.txt");
@@ -452,7 +505,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 
 		git.add().addFilepattern(untrackedFile.getName()).call();
 
-		git.reset().setRef(tagName).setMode(HARD).call();
+		assertSameAsHead(git.reset().setRef(tagName).setMode(HARD).call());
 
 		ObjectId head = db.resolve(Constants.HEAD);
 		assertEquals(secondCommit, head);
@@ -460,31 +513,34 @@ public class ResetCommandTest extends RepositoryTestCase {
 
 	@Test
 	public void testHardResetAfterSquashMerge() throws Exception {
-		Git g = new Git(db);
+		git = new Git(db);
 
 		writeTrashFile("file1", "file1");
-		g.add().addFilepattern("file1").call();
-		RevCommit first = g.commit().setMessage("initial commit").call();
+		git.add().addFilepattern("file1").call();
+		RevCommit first = git.commit().setMessage("initial commit").call();
 
 		assertTrue(new File(db.getWorkTree(), "file1").exists());
 		createBranch(first, "refs/heads/branch1");
 		checkoutBranch("refs/heads/branch1");
 
 		writeTrashFile("file2", "file2");
-		g.add().addFilepattern("file2").call();
-		g.commit().setMessage("second commit").call();
+		git.add().addFilepattern("file2").call();
+		git.commit().setMessage("second commit").call();
 		assertTrue(new File(db.getWorkTree(), "file2").exists());
 
 		checkoutBranch("refs/heads/master");
 
-		MergeResult result = g.merge().include(db.getRef("branch1"))
-				.setSquash(true).call();
+		MergeResult result = git.merge()
+				.include(db.exactRef("refs/heads/branch1"))
+				.setSquash(true)
+				.call();
 
 		assertEquals(MergeResult.MergeStatus.FAST_FORWARD_SQUASHED,
 				result.getMergeStatus());
 		assertNotNull(db.readSquashCommitMsg());
 
-		g.reset().setMode(ResetType.HARD).setRef(first.getName()).call();
+		assertSameAsHead(git.reset().setMode(ResetType.HARD)
+				.setRef(first.getName()).call());
 
 		assertNull(db.readSquashCommitMsg());
 	}
@@ -495,7 +551,7 @@ public class ResetCommandTest extends RepositoryTestCase {
 		File fileA = writeTrashFile("a.txt", "content");
 		git.add().addFilepattern("a.txt").call();
 		// Should assume an empty tree, like in C Git 1.8.2
-		git.reset().setMode(ResetType.HARD).call();
+		assertSameAsHead(git.reset().setMode(ResetType.HARD).call());
 
 		DirCache cache = db.readDirCache();
 		DirCacheEntry aEntry = cache.getEntry("a.txt");
@@ -528,6 +584,24 @@ public class ResetCommandTest extends RepositoryTestCase {
 				.getName());
 	}
 
+	private void assertReflogDisabled(ObjectId head)
+			throws IOException {
+		// Check the reflog for HEAD
+		String actualHeadMessage = db.getReflogReader(Constants.HEAD)
+				.getLastEntry().getComment();
+		String expectedHeadMessage = "commit: adding a.txt and dir/b.txt";
+		assertEquals(expectedHeadMessage, actualHeadMessage);
+		assertEquals(head.getName(), db.getReflogReader(Constants.HEAD)
+				.getLastEntry().getOldId().getName());
+
+		// The reflog for master contains the same as the one for HEAD
+		String actualMasterMessage = db.getReflogReader("refs/heads/master")
+				.getLastEntry().getComment();
+		String expectedMasterMessage = "commit: adding a.txt and dir/b.txt";
+		assertEquals(expectedMasterMessage, actualMasterMessage);
+		assertEquals(head.getName(), db.getReflogReader(Constants.HEAD)
+				.getLastEntry().getOldId().getName());
+	}
 	/**
 	 * Checks if a file with the given path exists in the HEAD tree
 	 *
@@ -537,16 +611,10 @@ public class ResetCommandTest extends RepositoryTestCase {
 	 */
 	private boolean inHead(String path) throws IOException {
 		ObjectId headId = db.resolve(Constants.HEAD);
-		RevWalk rw = new RevWalk(db);
-		TreeWalk tw = null;
-		try {
-			tw = TreeWalk.forPath(db, path, rw.parseTree(headId));
+		try (RevWalk rw = new RevWalk(db);
+				TreeWalk tw = TreeWalk.forPath(db, path,
+						rw.parseTree(headId))) {
 			return tw != null;
-		} finally {
-			rw.release();
-			rw.dispose();
-			if (tw != null)
-				tw.release();
 		}
 	}
 
@@ -562,4 +630,14 @@ public class ResetCommandTest extends RepositoryTestCase {
 		return dc.getEntry(path) != null;
 	}
 
+	/**
+	 * Asserts that a certain ref is similar to repos HEAD.
+	 * @param ref
+	 * @throws IOException
+	 */
+	private void assertSameAsHead(Ref ref) throws IOException {
+		Ref headRef = db.exactRef(Constants.HEAD);
+		assertEquals(headRef.getName(), ref.getName());
+		assertEquals(headRef.getObjectId(), ref.getObjectId());
+	}
 }
