@@ -93,6 +93,7 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 		return rng;
 	}
 
+	@Override
 	@Before
 	public void setUp() throws Exception {
 		super.setUp();
@@ -105,10 +106,11 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 		wc = (WindowCursor) repo.newObjectReader();
 	}
 
+	@Override
 	@After
 	public void tearDown() throws Exception {
 		if (wc != null)
-			wc.release();
+			wc.close();
 		new WindowCacheConfig().install();
 		super.tearDown();
 	}
@@ -143,7 +145,7 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 	public void testStandardFormat_LargeObject() throws Exception {
 		final int type = Constants.OBJ_BLOB;
 		byte[] data = getRng().nextBytes(streamThreshold + 5);
-		ObjectId id = new ObjectInserter.Formatter().idFor(type, data);
+		ObjectId id = getId(type, data);
 		write(id, compressStandardFormat(type, data));
 
 		ObjectLoader ol;
@@ -306,7 +308,7 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 			throws Exception {
 		final int type = Constants.OBJ_BLOB;
 		byte[] data = getRng().nextBytes(streamThreshold + 5);
-		ObjectId id = new ObjectInserter.Formatter().idFor(type, data);
+		ObjectId id = getId(type, data);
 		byte[] gz = compressStandardFormat(type, data);
 		gz[gz.length - 1] = 0;
 		gz[gz.length - 2] = 0;
@@ -344,7 +346,7 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 			throws Exception {
 		final int type = Constants.OBJ_BLOB;
 		byte[] data = getRng().nextBytes(streamThreshold + 5);
-		ObjectId id = new ObjectInserter.Formatter().idFor(type, data);
+		ObjectId id = getId(type, data);
 		byte[] gz = compressStandardFormat(type, data);
 		byte[] tr = new byte[gz.length - 1];
 		System.arraycopy(gz, 0, tr, 0, tr.length);
@@ -379,7 +381,7 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 			throws Exception {
 		final int type = Constants.OBJ_BLOB;
 		byte[] data = getRng().nextBytes(streamThreshold + 5);
-		ObjectId id = new ObjectInserter.Formatter().idFor(type, data);
+		ObjectId id = getId(type, data);
 		byte[] gz = compressStandardFormat(type, data);
 		byte[] tr = new byte[gz.length + 1];
 		System.arraycopy(gz, 0, tr, 0, gz.length);
@@ -438,7 +440,7 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 	public void testPackFormat_LargeObject() throws Exception {
 		final int type = Constants.OBJ_BLOB;
 		byte[] data = getRng().nextBytes(streamThreshold + 5);
-		ObjectId id = new ObjectInserter.Formatter().idFor(type, data);
+		ObjectId id = getId(type, data);
 		write(id, compressPackFormat(type, data));
 
 		ObjectLoader ol;
@@ -578,4 +580,10 @@ public class UnpackedObjectTest extends LocalDiskRepositoryTestCase {
 			out.close();
 		}
 	}
+
+	private ObjectId getId(int type, byte[] data) {
+		try (ObjectInserter.Formatter formatter = new ObjectInserter.Formatter()) {
+			return formatter.idFor(type, data);
+		}
+	}
 }
