@@ -119,36 +119,37 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 
 	@Test
 	public void addSubmodule() throws Exception {
-		Git git = new Git(db);
-		writeTrashFile("file.txt", "content");
-		git.add().addFilepattern("file.txt").call();
-		RevCommit commit = git.commit().setMessage("create file").call();
+		try (Git git = new Git(db)) {
+			writeTrashFile("file.txt", "content");
+			git.add().addFilepattern("file.txt").call();
+			RevCommit commit = git.commit().setMessage("create file").call();
 
-		SubmoduleAddCommand command = new SubmoduleAddCommand(db);
-		String path = "sub";
-		command.setPath(path);
-		String uri = db.getDirectory().toURI().toString();
-		command.setURI(uri);
-		Repository repo = command.call();
-		assertNotNull(repo);
-		ObjectId subCommit = repo.resolve(Constants.HEAD);
-		repo.close();
+			SubmoduleAddCommand command = new SubmoduleAddCommand(db);
+			String path = "sub";
+			command.setPath(path);
+			String uri = db.getDirectory().toURI().toString();
+			command.setURI(uri);
+			Repository repo = command.call();
+			assertNotNull(repo);
+			ObjectId subCommit = repo.resolve(Constants.HEAD);
+			repo.close();
 
-		SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
-		assertTrue(generator.next());
-		assertEquals(path, generator.getPath());
-		assertEquals(commit, generator.getObjectId());
-		assertEquals(uri, generator.getModulesUrl());
-		assertEquals(path, generator.getModulesPath());
-		assertEquals(uri, generator.getConfigUrl());
-		Repository subModRepo = generator.getRepository();
-		assertNotNull(subModRepo);
-		assertEquals(subCommit, commit);
-		subModRepo.close();
+			SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
+			assertTrue(generator.next());
+			assertEquals(path, generator.getPath());
+			assertEquals(commit, generator.getObjectId());
+			assertEquals(uri, generator.getModulesUrl());
+			assertEquals(path, generator.getModulesPath());
+			assertEquals(uri, generator.getConfigUrl());
+			Repository subModRepo = generator.getRepository();
+			assertNotNull(subModRepo);
+			assertEquals(subCommit, commit);
+			subModRepo.close();
 
-		Status status = Git.wrap(db).status().call();
-		assertTrue(status.getAdded().contains(Constants.DOT_GIT_MODULES));
-		assertTrue(status.getAdded().contains(path));
+			Status status = Git.wrap(db).status().call();
+			assertTrue(status.getAdded().contains(Constants.DOT_GIT_MODULES));
+			assertTrue(status.getAdded().contains(path));
+		}
 	}
 
 	@Test
@@ -160,6 +161,7 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 		DirCacheEditor editor = cache.editor();
 		editor.add(new PathEdit(path) {
 
+			@Override
 			public void apply(DirCacheEntry ent) {
 				ent.setFileMode(FileMode.GITLINK);
 				ent.setObjectId(id);
@@ -181,46 +183,79 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 	}
 
 	@Test
-	public void addSubmoduleWithRelativeUri() throws Exception {
-		Git git = new Git(db);
-		writeTrashFile("file.txt", "content");
-		git.add().addFilepattern("file.txt").call();
-		RevCommit commit = git.commit().setMessage("create file").call();
-
+	public void addSubmoduleWithInvalidPath() throws Exception {
 		SubmoduleAddCommand command = new SubmoduleAddCommand(db);
-		String path = "sub";
-		String uri = "./.git";
-		command.setPath(path);
-		command.setURI(uri);
-		Repository repo = command.call();
-		assertNotNull(repo);
-		addRepoToClose(repo);
+		command.setPath("-invalid-path");
+		// TODO(ms) set name to a valid value in 5.1.0 and adapt expected
+		// message below
+		command.setURI("http://example.com/repo/x.git");
+		try {
+			command.call().close();
+			fail("Exception not thrown");
+		} catch (IllegalArgumentException e) {
+			// TODO(ms) should check for submodule path, but can't set name
+			// before 5.1.0
+			assertEquals("Invalid submodule name '-invalid-path'",
+					e.getMessage());
+		}
+	}
 
-		SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
-		assertTrue(generator.next());
-		assertEquals(path, generator.getPath());
-		assertEquals(commit, generator.getObjectId());
-		assertEquals(uri, generator.getModulesUrl());
-		assertEquals(path, generator.getModulesPath());
-		String fullUri = db.getDirectory().getAbsolutePath();
-		if (File.separatorChar == '\\')
-			fullUri = fullUri.replace('\\', '/');
-		assertEquals(fullUri, generator.getConfigUrl());
-		Repository subModRepo = generator.getRepository();
-		assertNotNull(subModRepo);
-		assertEquals(
-				fullUri,
-				subModRepo
-						.getConfig()
-						.getString(ConfigConstants.CONFIG_REMOTE_SECTION,
-								Constants.DEFAULT_REMOTE_NAME,
-								ConfigConstants.CONFIG_KEY_URL));
-		subModRepo.close();
-		assertEquals(commit, repo.resolve(Constants.HEAD));
+	@Test
+	public void addSubmoduleWithInvalidUri() throws Exception {
+		SubmoduleAddCommand command = new SubmoduleAddCommand(db);
+		command.setPath("valid-path");
+		command.setURI("-upstream");
+		try {
+			command.call().close();
+			fail("Exception not thrown");
+		} catch (IllegalArgumentException e) {
+			assertEquals("Invalid submodule URL '-upstream'", e.getMessage());
+		}
+	}
 
-		Status status = Git.wrap(db).status().call();
-		assertTrue(status.getAdded().contains(Constants.DOT_GIT_MODULES));
-		assertTrue(status.getAdded().contains(path));
+	@Test
+	public void addSubmoduleWithRelativeUri() throws Exception {
+		try (Git git = new Git(db)) {
+			writeTrashFile("file.txt", "content");
+			git.add().addFilepattern("file.txt").call();
+			RevCommit commit = git.commit().setMessage("create file").call();
+
+			SubmoduleAddCommand command = new SubmoduleAddCommand(db);
+			String path = "sub";
+			String uri = "./.git";
+			command.setPath(path);
+			command.setURI(uri);
+			Repository repo = command.call();
+			assertNotNull(repo);
+			addRepoToClose(repo);
+
+			SubmoduleWalk generator = SubmoduleWalk.forIndex(db);
+			assertTrue(generator.next());
+			assertEquals(path, generator.getPath());
+			assertEquals(commit, generator.getObjectId());
+			assertEquals(uri, generator.getModulesUrl());
+			assertEquals(path, generator.getModulesPath());
+			String fullUri = db.getDirectory().getAbsolutePath();
+			if (File.separatorChar == '\\') {
+				fullUri = fullUri.replace('\\', '/');
+			}
+			assertEquals(fullUri, generator.getConfigUrl());
+			Repository subModRepo = generator.getRepository();
+			assertNotNull(subModRepo);
+			assertEquals(
+					fullUri,
+					subModRepo
+							.getConfig()
+							.getString(ConfigConstants.CONFIG_REMOTE_SECTION,
+									Constants.DEFAULT_REMOTE_NAME,
+									ConfigConstants.CONFIG_KEY_URL));
+			subModRepo.close();
+			assertEquals(commit, repo.resolve(Constants.HEAD));
+
+			Status status = Git.wrap(db).status().call();
+			assertTrue(status.getAdded().contains(Constants.DOT_GIT_MODULES));
+			assertTrue(status.getAdded().contains(path));
+		}
 	}
 
 	@Test
@@ -237,31 +272,32 @@ public class SubmoduleAddTest extends RepositoryTestCase {
 				path1, ConfigConstants.CONFIG_KEY_URL, url1);
 		modulesConfig.save();
 
-		Git git = new Git(db);
-		writeTrashFile("file.txt", "content");
-		git.add().addFilepattern("file.txt").call();
-		assertNotNull(git.commit().setMessage("create file").call());
+		try (Git git = new Git(db)) {
+			writeTrashFile("file.txt", "content");
+			git.add().addFilepattern("file.txt").call();
+			assertNotNull(git.commit().setMessage("create file").call());
 
-		SubmoduleAddCommand command = new SubmoduleAddCommand(db);
-		command.setPath(path2);
-		String url2 = db.getDirectory().toURI().toString();
-		command.setURI(url2);
-		Repository r = command.call();
-		assertNotNull(r);
-		addRepoToClose(r);
+			SubmoduleAddCommand command = new SubmoduleAddCommand(db);
+			command.setPath(path2);
+			String url2 = db.getDirectory().toURI().toString();
+			command.setURI(url2);
+			Repository r = command.call();
+			assertNotNull(r);
+			addRepoToClose(r);
 
-		modulesConfig.load();
-		assertEquals(path1, modulesConfig.getString(
-				ConfigConstants.CONFIG_SUBMODULE_SECTION, path1,
-				ConfigConstants.CONFIG_KEY_PATH));
-		assertEquals(url1, modulesConfig.getString(
-				ConfigConstants.CONFIG_SUBMODULE_SECTION, path1,
-				ConfigConstants.CONFIG_KEY_URL));
-		assertEquals(path2, modulesConfig.getString(
-				ConfigConstants.CONFIG_SUBMODULE_SECTION, path2,
-				ConfigConstants.CONFIG_KEY_PATH));
-		assertEquals(url2, modulesConfig.getString(
-				ConfigConstants.CONFIG_SUBMODULE_SECTION, path2,
-				ConfigConstants.CONFIG_KEY_URL));
+			modulesConfig.load();
+			assertEquals(path1, modulesConfig.getString(
+					ConfigConstants.CONFIG_SUBMODULE_SECTION, path1,
+					ConfigConstants.CONFIG_KEY_PATH));
+			assertEquals(url1, modulesConfig.getString(
+					ConfigConstants.CONFIG_SUBMODULE_SECTION, path1,
+					ConfigConstants.CONFIG_KEY_URL));
+			assertEquals(path2, modulesConfig.getString(
+					ConfigConstants.CONFIG_SUBMODULE_SECTION, path2,
+					ConfigConstants.CONFIG_KEY_PATH));
+			assertEquals(url2, modulesConfig.getString(
+					ConfigConstants.CONFIG_SUBMODULE_SECTION, path2,
+					ConfigConstants.CONFIG_KEY_URL));
+		}
 	}
-}
\ No newline at end of file
+}
