@@ -83,8 +83,8 @@ public class PushProcessTest extends SampleDataRepositoryTestCase {
 	public void setUp() throws Exception {
 		super.setUp();
 		transport = new MockTransport(db, new URIish());
-		refUpdates = new HashSet<RemoteRefUpdate>();
-		advertisedRefs = new HashSet<Ref>();
+		refUpdates = new HashSet<>();
+		advertisedRefs = new HashSet<>();
 		connectionUpdateStatus = Status.OK;
 	}
 
@@ -421,7 +421,7 @@ public class PushProcessTest extends SampleDataRepositoryTestCase {
 	private class MockPushConnection extends BaseConnection implements
 			PushConnection {
 		MockPushConnection() {
-			final Map<String, Ref> refsMap = new HashMap<String, Ref>();
+			final Map<String, Ref> refsMap = new HashMap<>();
 			for (final Ref r : advertisedRefs)
 				refsMap.put(r.getName(), r);
 			available(refsMap);
@@ -432,12 +432,14 @@ public class PushProcessTest extends SampleDataRepositoryTestCase {
 			// nothing here
 		}
 
+		@Override
 		public void push(ProgressMonitor monitor,
 				Map<String, RemoteRefUpdate> refsToUpdate, OutputStream out)
 				throws TransportException {
 			push(monitor, refsToUpdate);
 		}
 
+		@Override
 		public void push(ProgressMonitor monitor,
 				Map<String, RemoteRefUpdate> refsToUpdate)
 				throws TransportException {
