@@ -53,7 +53,9 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
+
 import org.eclipse.jgit.junit.TestRng;
+import org.eclipse.jgit.util.TemporaryBuffer.Block;
 import org.junit.Test;
 
 public class TemporaryBufferTest {
@@ -424,4 +426,27 @@ public class TemporaryBufferTest {
 			assertEquals("In-memory buffer limit exceeded", e.getMessage());
 		}
 	}
+
+	@Test
+	public void testHeapWithEstimatedSize() throws IOException {
+		int sz = 2 * Block.SZ;
+		try (TemporaryBuffer b = new TemporaryBuffer.Heap(sz / 2, sz)) {
+			for (int i = 0; i < sz; i++) {
+				b.write('x');
+			}
+			try {
+				b.write(1);
+				fail("accepted too many bytes of data");
+			} catch (IOException e) {
+				assertEquals("In-memory buffer limit exceeded", e.getMessage());
+			}
+
+			try (InputStream in = b.openInputStream()) {
+				for (int i = 0; i < sz; i++) {
+					assertEquals('x', in.read());
+				}
+				assertEquals(-1, in.read());
+			}
+		}
+	}
 }
