@@ -46,6 +46,11 @@ import java.io.Closeable;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.text.MessageFormat;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 
@@ -60,23 +65,21 @@ import org.eclipse.jgit.lib.ObjectReader;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.treewalk.TreeWalk;
+import org.eclipse.jgit.treewalk.filter.PathFilterGroup;
 
 /**
  * Create an archive of files from a named tree.
  * <p>
- * Examples (<code>git</code> is a {@link Git} instance):
+ * Examples (<code>git</code> is a {@link org.eclipse.jgit.api.Git} instance):
  * <p>
  * Create a tarball from HEAD:
  *
  * <pre>
  * ArchiveCommand.registerFormat("tar", new TarFormat());
  * try {
- *	git.archive()
- *		.setTree(db.resolve(&quot;HEAD&quot;))
- *		.setOutputStream(out)
- *		.call();
+ * 	git.archive().setTree(db.resolve(&quot;HEAD&quot;)).setOutputStream(out).call();
  * } finally {
- *	ArchiveCommand.unregisterFormat("tar");
+ * 	ArchiveCommand.unregisterFormat("tar");
  * }
  * </pre>
  * <p>
@@ -97,7 +100,6 @@ import org.eclipse.jgit.treewalk.TreeWalk;
  *
  * @see <a href="http://git-htmldocs.googlecode.com/git/git-archive.html" >Git
  *      documentation about archive</a>
- *
  * @since 3.1
  */
 public class ArchiveCommand extends GitCommand<OutputStream> {
@@ -131,14 +133,33 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 		 */
 		T createArchiveOutputStream(OutputStream s) throws IOException;
 
+		/**
+		 * Start a new archive. Entries can be included in the archive using the
+		 * putEntry method, and then the archive should be closed using its
+		 * close method. In addition options can be applied to the underlying
+		 * stream. E.g. compression level.
+		 *
+		 * @param s
+		 *            underlying output stream to which to write the archive.
+		 * @param o
+		 *            options to apply to the underlying output stream. Keys are
+		 *            option names and values are option values.
+		 * @return new archive object for use in putEntry
+		 * @throws IOException
+		 *             thrown by the underlying output stream for I/O errors
+		 * @since 4.0
+		 */
+		T createArchiveOutputStream(OutputStream s, Map<String, Object> o)
+				throws IOException;
+
 		/**
 		 * Write an entry to an archive.
 		 *
 		 * @param out
 		 *            archive object from createArchiveOutputStream
 		 * @param path
-		 *            full filename relative to the root of the archive
-		 *            (with trailing '/' for directories)
+		 *            full filename relative to the root of the archive (with
+		 *            trailing '/' for directories)
 		 * @param mode
 		 *            mode (for example FileMode.REGULAR_FILE or
 		 *            FileMode.SYMLINK)
@@ -146,9 +167,36 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 		 *            blob object with data for this entry (null for
 		 *            directories)
 		 * @throws IOException
-		 *            thrown by the underlying output stream for I/O errors
+		 *             thrown by the underlying output stream for I/O errors
+		 * @deprecated use
+		 *             {@link #putEntry(Closeable, ObjectId, String, FileMode, ObjectLoader)}
+		 *             instead
 		 */
+		@Deprecated
 		void putEntry(T out, String path, FileMode mode,
+					  ObjectLoader loader) throws IOException;
+
+		/**
+		 * Write an entry to an archive.
+		 *
+		 * @param out
+		 *            archive object from createArchiveOutputStream
+		 * @param tree
+		 *            the tag, commit, or tree object to produce an archive for
+		 * @param path
+		 *            full filename relative to the root of the archive (with
+		 *            trailing '/' for directories)
+		 * @param mode
+		 *            mode (for example FileMode.REGULAR_FILE or
+		 *            FileMode.SYMLINK)
+		 * @param loader
+		 *            blob object with data for this entry (null for
+		 *            directories)
+		 * @throws IOException
+		 *             thrown by the underlying output stream for I/O errors
+		 * @since 4.7
+		 */
+		void putEntry(T out, ObjectId tree, String path, FileMode mode,
 				ObjectLoader loader) throws IOException;
 
 		/**
@@ -189,77 +237,152 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 		}
 	}
 
+	private static class FormatEntry {
+		final Format<?> format;
+		/** Number of times this format has been registered. */
+		final int refcnt;
+
+		public FormatEntry(Format<?> format, int refcnt) {
+			if (format == null)
+				throw new NullPointerException();
+			this.format = format;
+			this.refcnt = refcnt;
+		}
+	}
+
 	/**
 	 * Available archival formats (corresponding to values for
 	 * the --format= option)
 	 */
-	private static final ConcurrentMap<String, Format<?>> formats =
-			new ConcurrentHashMap<String, Format<?>>();
+	private static final ConcurrentMap<String, FormatEntry> formats =
+			new ConcurrentHashMap<>();
+
+	/**
+	 * Replaces the entry for a key only if currently mapped to a given
+	 * value.
+	 *
+	 * @param map a map
+	 * @param key key with which the specified value is associated
+	 * @param oldValue expected value for the key (null if should be absent).
+	 * @param newValue value to be associated with the key (null to remove).
+	 * @return true if the value was replaced
+	 */
+	private static <K, V> boolean replace(ConcurrentMap<K, V> map,
+			K key, V oldValue, V newValue) {
+		if (oldValue == null && newValue == null) // Nothing to do.
+			return true;
+
+		if (oldValue == null)
+			return map.putIfAbsent(key, newValue) == null;
+		else if (newValue == null)
+			return map.remove(key, oldValue);
+		else
+			return map.replace(key, oldValue, newValue);
+	}
 
 	/**
 	 * Adds support for an additional archival format.  To avoid
 	 * unnecessary dependencies, ArchiveCommand does not have support
 	 * for any formats built in; use this function to add them.
-	 *
+	 * <p>
 	 * OSGi plugins providing formats should call this function at
 	 * bundle activation time.
+	 * <p>
+	 * It is okay to register the same archive format with the same
+	 * name multiple times, but don't forget to unregister it that
+	 * same number of times, too.
+	 * <p>
+	 * Registering multiple formats with different names and the
+	 * same or overlapping suffixes results in undefined behavior.
+	 * TODO: check that suffixes don't overlap.
 	 *
 	 * @param name name of a format (e.g., "tar" or "zip").
 	 * @param fmt archiver for that format
 	 * @throws JGitInternalException
-	 *              An archival format with that name was already registered.
+	 *              A different archival format with that name was
+	 *              already registered.
 	 */
 	public static void registerFormat(String name, Format<?> fmt) {
-		// TODO(jrn): Check that suffixes don't overlap.
+		if (fmt == null)
+			throw new NullPointerException();
 
-		if (formats.putIfAbsent(name, fmt) != null)
-			throw new JGitInternalException(MessageFormat.format(
-					JGitText.get().archiveFormatAlreadyRegistered,
-					name));
+		FormatEntry old, entry;
+		do {
+			old = formats.get(name);
+			if (old == null) {
+				entry = new FormatEntry(fmt, 1);
+				continue;
+			}
+			if (!old.format.equals(fmt))
+				throw new JGitInternalException(MessageFormat.format(
+						JGitText.get().archiveFormatAlreadyRegistered,
+						name));
+			entry = new FormatEntry(old.format, old.refcnt + 1);
+		} while (!replace(formats, name, old, entry));
 	}
 
 	/**
-	 * Removes support for an archival format so its Format can be
-	 * garbage collected.
+	 * Marks support for an archival format as no longer needed so its
+	 * Format can be garbage collected if no one else is using it either.
+	 * <p>
+	 * In other words, this decrements the reference count for an
+	 * archival format.  If the reference count becomes zero, removes
+	 * support for that format.
 	 *
 	 * @param name name of format (e.g., "tar" or "zip").
 	 * @throws JGitInternalException
 	 *              No such archival format was registered.
 	 */
 	public static void unregisterFormat(String name) {
-		if (formats.remove(name) == null)
-			throw new JGitInternalException(MessageFormat.format(
-					JGitText.get().archiveFormatAlreadyAbsent,
-					name));
+		FormatEntry old, entry;
+		do {
+			old = formats.get(name);
+			if (old == null)
+				throw new JGitInternalException(MessageFormat.format(
+						JGitText.get().archiveFormatAlreadyAbsent,
+						name));
+			if (old.refcnt == 1) {
+				entry = null;
+				continue;
+			}
+			entry = new FormatEntry(old.format, old.refcnt - 1);
+		} while (!replace(formats, name, old, entry));
 	}
 
 	private static Format<?> formatBySuffix(String filenameSuffix)
 			throws UnsupportedFormatException {
 		if (filenameSuffix != null)
-			for (Format<?> fmt : formats.values())
+			for (FormatEntry entry : formats.values()) {
+				Format<?> fmt = entry.format;
 				for (String sfx : fmt.suffixes())
 					if (filenameSuffix.endsWith(sfx))
 						return fmt;
+			}
 		return lookupFormat("tar"); //$NON-NLS-1$
 	}
 
 	private static Format<?> lookupFormat(String formatName) throws UnsupportedFormatException {
-		Format<?> fmt = formats.get(formatName);
-		if (fmt == null)
+		FormatEntry entry = formats.get(formatName);
+		if (entry == null)
 			throw new UnsupportedFormatException(formatName);
-		return fmt;
+		return entry.format;
 	}
 
 	private OutputStream out;
 	private ObjectId tree;
 	private String prefix;
 	private String format;
+	private Map<String, Object> formatOptions = new HashMap<>();
+	private List<String> paths = new ArrayList<>();
 
 	/** Filename suffix, for automatically choosing a format. */
 	private String suffix;
 
 	/**
+	 * Constructor for ArchiveCommand
+	 *
 	 * @param repo
+	 *            the {@link org.eclipse.jgit.lib.Repository}
 	 */
 	public ArchiveCommand(Repository repo) {
 		super(repo);
@@ -267,18 +390,27 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 	}
 
 	private <T extends Closeable> OutputStream writeArchive(Format<T> fmt) {
-		final String pfx = prefix == null ? "" : prefix; //$NON-NLS-1$
-		final TreeWalk walk = new TreeWalk(repo);
 		try {
-			final T outa = fmt.createArchiveOutputStream(out);
-			try {
-				final MutableObjectId idBuf = new MutableObjectId();
-				final ObjectReader reader = walk.getObjectReader();
-				final RevWalk rw = new RevWalk(walk.getObjectReader());
+			try (TreeWalk walk = new TreeWalk(repo);
+					RevWalk rw = new RevWalk(walk.getObjectReader());
+					T outa = fmt.createArchiveOutputStream(out,
+							formatOptions)) {
+				String pfx = prefix == null ? "" : prefix; //$NON-NLS-1$
+				MutableObjectId idBuf = new MutableObjectId();
+				ObjectReader reader = walk.getObjectReader();
 
 				walk.reset(rw.parseTree(tree));
+				if (!paths.isEmpty())
+					walk.setFilter(PathFilterGroup.createFromStrings(paths));
+
+				// Put base directory into archive
+				if (pfx.endsWith("/")) { //$NON-NLS-1$
+					fmt.putEntry(outa, tree, pfx.replaceAll("[/]+$", "/"), //$NON-NLS-1$ //$NON-NLS-2$
+							FileMode.TREE, null);
+				}
+
 				while (walk.next()) {
-					final String name = pfx + walk.getPathString();
+					String name = pfx + walk.getPathString();
 					FileMode mode = walk.getFileMode(0);
 
 					if (walk.isSubtree())
@@ -290,34 +422,29 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 						mode = FileMode.TREE;
 
 					if (mode == FileMode.TREE) {
-						fmt.putEntry(outa, name + "/", mode, null);
+						fmt.putEntry(outa, tree, name + "/", mode, null); //$NON-NLS-1$
 						continue;
 					}
 					walk.getObjectId(idBuf, 0);
-					fmt.putEntry(outa, name, mode, reader.open(idBuf));
+					fmt.putEntry(outa, tree, name, mode, reader.open(idBuf));
 				}
-				outa.close();
+				return out;
 			} finally {
 				out.close();
 			}
-			return out;
 		} catch (IOException e) {
 			// TODO(jrn): Throw finer-grained errors.
 			throw new JGitInternalException(
 					JGitText.get().exceptionCaughtDuringExecutionOfArchiveCommand, e);
-		} finally {
-			walk.release();
 		}
 	}
 
-	/**
-	 * @return the stream to which the archive has been written
-	 */
+	/** {@inheritDoc} */
 	@Override
 	public OutputStream call() throws GitAPIException {
 		checkCallable();
 
-		final Format<?> fmt;
+		Format<?> fmt;
 		if (format == null)
 			fmt = formatBySuffix(suffix);
 		else
@@ -326,6 +453,8 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 	}
 
 	/**
+	 * Set the tag, commit, or tree object to produce an archive for
+	 *
 	 * @param tree
 	 *            the tag, commit, or tree object to produce an archive for
 	 * @return this
@@ -340,6 +469,8 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 	}
 
 	/**
+	 * Set string prefixed to filenames in archive
+	 *
 	 * @param prefix
 	 *            string prefixed to filenames in archive (e.g., "master/").
 	 *            null means to not use any leading prefix.
@@ -372,8 +503,10 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 	}
 
 	/**
+	 * Set output stream
+	 *
 	 * @param out
-	 *	      the stream to which to write the archive
+	 *            the stream to which to write the archive
 	 * @return this
 	 */
 	public ArchiveCommand setOutputStream(OutputStream out) {
@@ -382,14 +515,47 @@ public class ArchiveCommand extends GitCommand<OutputStream> {
 	}
 
 	/**
+	 * Set archive format
+	 *
 	 * @param fmt
-	 *	      archive format (e.g., "tar" or "zip").
-	 *	      null means to choose automatically based on
-	 *	      the archive filename.
+	 *            archive format (e.g., "tar" or "zip"). null means to choose
+	 *            automatically based on the archive filename.
 	 * @return this
 	 */
 	public ArchiveCommand setFormat(String fmt) {
 		this.format = fmt;
 		return this;
 	}
+
+	/**
+	 * Set archive format options
+	 *
+	 * @param options
+	 *            archive format options (e.g., level=9 for zip compression).
+	 * @return this
+	 * @since 4.0
+	 */
+	public ArchiveCommand setFormatOptions(Map<String, Object> options) {
+		this.formatOptions = options;
+		return this;
+	}
+
+	/**
+	 * Set an optional parameter path. without an optional path parameter, all
+	 * files and subdirectories of the current working directory are included in
+	 * the archive. If one or more paths are specified, only these are included.
+	 *
+	 * @param paths
+	 *            file names (e.g <code>file1.c</code>) or directory names (e.g.
+	 *            <code>dir</code> to add <code>dir/file1</code> and
+	 *            <code>dir/file2</code>) can also be given to add all files in
+	 *            the directory, recursively. Fileglobs (e.g. *.c) are not yet
+	 *            supported.
+	 * @return this
+	 * @since 3.4
+	 */
+	public ArchiveCommand setPaths(String... paths) {
+		this.paths = Arrays.asList(paths);
+		return this;
+	}
 }
