@@ -1,7 +1,5 @@
 package org.eclipse.jgit.api.errors;
 
-import org.eclipse.jgit.api.errors.GitAPIException;
-
 /**
  * Thrown from StashApplyCommand when stash apply fails
  */
@@ -9,10 +7,24 @@ public class StashApplyFailureException extends GitAPIException {
 
 	private static final long serialVersionUID = 1L;
 
+	/**
+	 * Constructor for StashApplyFailureException
+	 *
+	 * @param message
+	 *            error message
+	 * @param cause
+	 *            a {@link java.lang.Throwable}
+	 * @since 4.1
+	 */
+	public StashApplyFailureException(String message, Throwable cause) {
+		super(message, cause);
+	}
+
 	/**
 	 * Create a StashApplyFailedException
 	 *
 	 * @param message
+	 *            error message
 	 */
 	public StashApplyFailureException(final String message) {
 		super(message);
