@@ -44,19 +44,18 @@
 
 package org.eclipse.jgit.internal.storage.dfs;
 
+import static org.eclipse.jgit.internal.storage.dfs.DfsObjDatabase.PackSource.UNREACHABLE_GARBAGE;
 import static org.eclipse.jgit.internal.storage.pack.PackExt.PACK;
 import static org.eclipse.jgit.lib.Constants.OBJECT_ID_LENGTH;
 
 import java.io.IOException;
-import java.security.MessageDigest;
-import java.text.MessageFormat;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Set;
 import java.util.zip.DataFormatException;
@@ -66,6 +65,8 @@ import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.errors.StoredObjectRepresentationNotAvailableException;
 import org.eclipse.jgit.internal.JGitText;
+import org.eclipse.jgit.internal.storage.dfs.DfsObjDatabase.PackList;
+import org.eclipse.jgit.internal.storage.dfs.DfsObjDatabase.PackSource;
 import org.eclipse.jgit.internal.storage.file.BitmapIndexImpl;
 import org.eclipse.jgit.internal.storage.file.PackBitmapIndex;
 import org.eclipse.jgit.internal.storage.file.PackIndex;
@@ -81,7 +82,6 @@ import org.eclipse.jgit.lib.AsyncObjectLoaderQueue;
 import org.eclipse.jgit.lib.AsyncObjectSizeQueue;
 import org.eclipse.jgit.lib.BitmapIndex;
 import org.eclipse.jgit.lib.BitmapIndex.BitmapBuilder;
-import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.InflaterCache;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.ObjectLoader;
@@ -92,28 +92,35 @@ import org.eclipse.jgit.util.BlockList;
 /**
  * Reader to access repository content through.
  * <p>
- * See the base {@link ObjectReader} documentation for details. Notably, a
- * reader is not thread safe.
+ * See the base {@link org.eclipse.jgit.lib.ObjectReader} documentation for
+ * details. Notably, a reader is not thread safe.
  */
-public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
+public class DfsReader extends ObjectReader implements ObjectReuseAsIs {
+	private static final int MAX_RESOLVE_MATCHES = 256;
+
 	/** Temporary buffer large enough for at least one raw object id. */
 	final byte[] tempId = new byte[OBJECT_ID_LENGTH];
 
 	/** Database this reader loads objects from. */
 	final DfsObjDatabase db;
 
+	final DfsReaderIoStats.Accumulator stats = new DfsReaderIoStats.Accumulator();
+
 	private Inflater inf;
-
 	private DfsBlock block;
-
 	private DeltaBaseCache baseCache;
-
 	private DfsPackFile last;
-
 	private boolean avoidUnreachable;
 
-	DfsReader(DfsObjDatabase db) {
+	/**
+	 * Initialize a new DfsReader
+	 *
+	 * @param db
+	 *            parent DfsObjDatabase.
+	 */
+	protected DfsReader(DfsObjDatabase db) {
 		this.db = db;
+		this.streamFileThreshold = db.getReaderOptions().getStreamFileThreshold();
 	}
 
 	DfsReaderOptions getOptions() {
@@ -126,20 +133,19 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		return baseCache;
 	}
 
-	int getStreamFileThreshold() {
-		return getOptions().getStreamFileThreshold();
-	}
-
+	/** {@inheritDoc} */
 	@Override
 	public ObjectReader newReader() {
-		return new DfsReader(db);
+		return db.newReader();
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public void setAvoidUnreachableObjects(boolean avoid) {
 		avoidUnreachable = avoid;
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public BitmapIndex getBitmapIndex() throws IOException {
 		for (DfsPackFile pack : db.getPacks()) {
@@ -150,6 +156,8 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		return null;
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public Collection<CachedPack> getCachedPacksAndUpdate(
 		BitmapBuilder needBitmap) throws IOException {
 		for (DfsPackFile pack : db.getPacks()) {
@@ -161,30 +169,56 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		return Collections.emptyList();
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public Collection<ObjectId> resolve(AbbreviatedObjectId id)
 			throws IOException {
 		if (id.isComplete())
 			return Collections.singleton(id.toObjectId());
-		boolean noGarbage = avoidUnreachable;
-		HashSet<ObjectId> matches = new HashSet<ObjectId>(4);
-		for (DfsPackFile pack : db.getPacks()) {
-			if (noGarbage && pack.isGarbage())
-				continue;
-			pack.resolve(this, matches, id, 256);
-			if (256 <= matches.size())
-				break;
+		HashSet<ObjectId> matches = new HashSet<>(4);
+		PackList packList = db.getPackList();
+		resolveImpl(packList, id, matches);
+		if (matches.size() < MAX_RESOLVE_MATCHES && packList.dirty()) {
+			stats.scanPacks++;
+			resolveImpl(db.scanPacks(packList), id, matches);
 		}
 		return matches;
 	}
 
+	private void resolveImpl(PackList packList, AbbreviatedObjectId id,
+			HashSet<ObjectId> matches) throws IOException {
+		for (DfsPackFile pack : packList.packs) {
+			if (skipGarbagePack(pack)) {
+				continue;
+			}
+			pack.resolve(this, matches, id, MAX_RESOLVE_MATCHES);
+			if (matches.size() >= MAX_RESOLVE_MATCHES) {
+				break;
+			}
+		}
+	}
+
+	/** {@inheritDoc} */
 	@Override
 	public boolean has(AnyObjectId objectId) throws IOException {
-		if (last != null && last.hasObject(this, objectId))
+		if (last != null
+				&& !skipGarbagePack(last)
+				&& last.hasObject(this, objectId))
 			return true;
-		boolean noGarbage = avoidUnreachable;
-		for (DfsPackFile pack : db.getPacks()) {
-			if (pack == last || (noGarbage && pack.isGarbage()))
+		PackList packList = db.getPackList();
+		if (hasImpl(packList, objectId)) {
+			return true;
+		} else if (packList.dirty()) {
+			stats.scanPacks++;
+			return hasImpl(db.scanPacks(packList), objectId);
+		}
+		return false;
+	}
+
+	private boolean hasImpl(PackList packList, AnyObjectId objectId)
+			throws IOException {
+		for (DfsPackFile pack : packList.packs) {
+			if (pack == last || skipGarbagePack(pack))
 				continue;
 			if (pack.hasObject(this, objectId)) {
 				last = pack;
@@ -194,38 +228,69 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		return false;
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public ObjectLoader open(AnyObjectId objectId, int typeHint)
 			throws MissingObjectException, IncorrectObjectTypeException,
 			IOException {
-		if (last != null) {
-			ObjectLoader ldr = last.get(this, objectId);
-			if (ldr != null)
-				return ldr;
+		ObjectLoader ldr;
+		if (last != null && !skipGarbagePack(last)) {
+			ldr = last.get(this, objectId);
+			if (ldr != null) {
+				return checkType(ldr, objectId, typeHint);
+			}
 		}
 
-		boolean noGarbage = avoidUnreachable;
-		for (DfsPackFile pack : db.getPacks()) {
-			if (pack == last || (noGarbage && pack.isGarbage()))
+		PackList packList = db.getPackList();
+		ldr = openImpl(packList, objectId);
+		if (ldr != null) {
+			return checkType(ldr, objectId, typeHint);
+		}
+		if (packList.dirty()) {
+			stats.scanPacks++;
+			ldr = openImpl(db.scanPacks(packList), objectId);
+			if (ldr != null) {
+				return checkType(ldr, objectId, typeHint);
+			}
+		}
+
+		if (typeHint == OBJ_ANY)
+			throw new MissingObjectException(objectId.copy(),
+					JGitText.get().unknownObjectType2);
+		throw new MissingObjectException(objectId.copy(), typeHint);
+	}
+
+	private static ObjectLoader checkType(ObjectLoader ldr, AnyObjectId id,
+			int typeHint) throws IncorrectObjectTypeException {
+		if (typeHint != OBJ_ANY && ldr.getType() != typeHint) {
+			throw new IncorrectObjectTypeException(id.copy(), typeHint);
+		}
+		return ldr;
+	}
+
+	private ObjectLoader openImpl(PackList packList, AnyObjectId objectId)
+			throws IOException {
+		for (DfsPackFile pack : packList.packs) {
+			if (pack == last || skipGarbagePack(pack)) {
 				continue;
+			}
 			ObjectLoader ldr = pack.get(this, objectId);
 			if (ldr != null) {
 				last = pack;
 				return ldr;
 			}
 		}
-
-		if (typeHint == OBJ_ANY)
-			throw new MissingObjectException(objectId.copy(), "unknown");
-		throw new MissingObjectException(objectId.copy(), typeHint);
+		return null;
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public Set<ObjectId> getShallowCommits() {
 		return Collections.emptySet();
 	}
 
 	private static final Comparator<FoundObject<?>> FOUND_OBJECT_SORT = new Comparator<FoundObject<?>>() {
+		@Override
 		public int compare(FoundObject<?> a, FoundObject<?> b) {
 			int cmp = a.packIndex - b.packIndex;
 			if (cmp == 0)
@@ -257,39 +322,60 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 
 	private <T extends ObjectId> Iterable<FoundObject<T>> findAll(
 			Iterable<T> objectIds) throws IOException {
-		ArrayList<FoundObject<T>> r = new ArrayList<FoundObject<T>>();
-		DfsPackFile[] packList = db.getPacks();
-		if (packList.length == 0) {
-			for (T t : objectIds)
-				r.add(new FoundObject<T>(t));
-			return r;
+		Collection<T> pending = new LinkedList<>();
+		for (T id : objectIds) {
+			pending.add(id);
 		}
 
-		int lastIdx = 0;
-		DfsPackFile lastPack = packList[lastIdx];
-		boolean noGarbage = avoidUnreachable;
+		PackList packList = db.getPackList();
+		List<FoundObject<T>> r = new ArrayList<>();
+		findAllImpl(packList, pending, r);
+		if (!pending.isEmpty() && packList.dirty()) {
+			stats.scanPacks++;
+			findAllImpl(db.scanPacks(packList), pending, r);
+		}
+		for (T t : pending) {
+			r.add(new FoundObject<>(t));
+		}
+		Collections.sort(r, FOUND_OBJECT_SORT);
+		return r;
+	}
 
-		OBJECT_SCAN: for (T t : objectIds) {
-			try {
-				long p = lastPack.findOffset(this, t);
-				if (0 < p) {
-					r.add(new FoundObject<T>(t, lastIdx, lastPack, p));
-					continue;
+	private <T extends ObjectId> void findAllImpl(PackList packList,
+			Collection<T> pending, List<FoundObject<T>> r) {
+		DfsPackFile[] packs = packList.packs;
+		if (packs.length == 0) {
+			return;
+		}
+		int lastIdx = 0;
+		DfsPackFile lastPack = packs[lastIdx];
+
+		OBJECT_SCAN: for (Iterator<T> it = pending.iterator(); it.hasNext();) {
+			T t = it.next();
+			if (!skipGarbagePack(lastPack)) {
+				try {
+					long p = lastPack.findOffset(this, t);
+					if (0 < p) {
+						r.add(new FoundObject<>(t, lastIdx, lastPack, p));
+						it.remove();
+						continue;
+					}
+				} catch (IOException e) {
+					// Fall though and try to examine other packs.
 				}
-			} catch (IOException e) {
-				// Fall though and try to examine other packs.
 			}
 
-			for (int i = 0; i < packList.length; i++) {
+			for (int i = 0; i < packs.length; i++) {
 				if (i == lastIdx)
 					continue;
-				DfsPackFile pack = packList[i];
-				if (noGarbage && pack.isGarbage())
+				DfsPackFile pack = packs[i];
+				if (skipGarbagePack(pack))
 					continue;
 				try {
 					long p = pack.findOffset(this, t);
 					if (0 < p) {
-						r.add(new FoundObject<T>(t, i, pack, p));
+						r.add(new FoundObject<>(t, i, pack, p));
+						it.remove();
 						lastIdx = i;
 						lastPack = pack;
 						continue OBJECT_SCAN;
@@ -298,15 +384,16 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 					// Examine other packs.
 				}
 			}
-
-			r.add(new FoundObject<T>(t));
 		}
 
-		Collections.sort(r, FOUND_OBJECT_SORT);
 		last = lastPack;
-		return r;
 	}
 
+	private boolean skipGarbagePack(DfsPackFile pack) {
+		return avoidUnreachable && pack.isGarbage();
+	}
+
+	/** {@inheritDoc} */
 	@Override
 	public <T extends ObjectId> AsyncObjectLoaderQueue<T> open(
 			Iterable<T> objectIds, final boolean reportMissing) {
@@ -324,6 +411,7 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		return new AsyncObjectLoaderQueue<T>() {
 			private FoundObject<T> cur;
 
+			@Override
 			public boolean next() throws MissingObjectException, IOException {
 				if (idItr.hasNext()) {
 					cur = idItr.next();
@@ -335,30 +423,37 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 				}
 			}
 
+			@Override
 			public T getCurrent() {
 				return cur.id;
 			}
 
+			@Override
 			public ObjectId getObjectId() {
 				return cur.id;
 			}
 
+			@Override
 			public ObjectLoader open() throws IOException {
 				if (cur.pack == null)
-					throw new MissingObjectException(cur.id, "unknown");
+					throw new MissingObjectException(cur.id,
+							JGitText.get().unknownObjectType2);
 				return cur.pack.load(DfsReader.this, cur.offset);
 			}
 
+			@Override
 			public boolean cancel(boolean mayInterruptIfRunning) {
 				return true;
 			}
 
+			@Override
 			public void release() {
 				// Nothing to clean up.
 			}
 		};
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public <T extends ObjectId> AsyncObjectSizeQueue<T> getObjectSize(
 			Iterable<T> objectIds, final boolean reportMissing) {
@@ -375,14 +470,15 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		final IOException findAllError = error;
 		return new AsyncObjectSizeQueue<T>() {
 			private FoundObject<T> cur;
-
 			private long sz;
 
+			@Override
 			public boolean next() throws MissingObjectException, IOException {
 				if (idItr.hasNext()) {
 					cur = idItr.next();
 					if (cur.pack == null)
-						throw new MissingObjectException(cur.id, "unknown");
+						throw new MissingObjectException(cur.id,
+								JGitText.get().unknownObjectType2);
 					sz = cur.pack.getObjectSize(DfsReader.this, cur.offset);
 					return true;
 				} else if (findAllError != null) {
@@ -392,68 +488,112 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 				}
 			}
 
+			@Override
 			public T getCurrent() {
 				return cur.id;
 			}
 
+			@Override
 			public ObjectId getObjectId() {
 				return cur.id;
 			}
 
+			@Override
 			public long getSize() {
 				return sz;
 			}
 
+			@Override
 			public boolean cancel(boolean mayInterruptIfRunning) {
 				return true;
 			}
 
+			@Override
 			public void release() {
 				// Nothing to clean up.
 			}
 		};
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public long getObjectSize(AnyObjectId objectId, int typeHint)
 			throws MissingObjectException, IncorrectObjectTypeException,
 			IOException {
-		if (last != null) {
+		if (last != null && !skipGarbagePack(last)) {
 			long sz = last.getObjectSize(this, objectId);
-			if (0 <= sz)
+			if (0 <= sz) {
 				return sz;
+			}
 		}
 
-		for (DfsPackFile pack : db.getPacks()) {
-			if (pack == last)
+		PackList packList = db.getPackList();
+		long sz = getObjectSizeImpl(packList, objectId);
+		if (0 <= sz) {
+			return sz;
+		}
+		if (packList.dirty()) {
+			sz = getObjectSizeImpl(packList, objectId);
+			if (0 <= sz) {
+				return sz;
+			}
+		}
+
+		if (typeHint == OBJ_ANY) {
+			throw new MissingObjectException(objectId.copy(),
+					JGitText.get().unknownObjectType2);
+		}
+		throw new MissingObjectException(objectId.copy(), typeHint);
+	}
+
+	private long getObjectSizeImpl(PackList packList, AnyObjectId objectId)
+			throws IOException {
+		for (DfsPackFile pack : packList.packs) {
+			if (pack == last || skipGarbagePack(pack)) {
 				continue;
+			}
 			long sz = pack.getObjectSize(this, objectId);
 			if (0 <= sz) {
 				last = pack;
 				return sz;
 			}
 		}
-
-		if (typeHint == OBJ_ANY)
-			throw new MissingObjectException(objectId.copy(), "unknown");
-		throw new MissingObjectException(objectId.copy(), typeHint);
+		return -1;
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public DfsObjectToPack newObjectToPack(AnyObjectId objectId, int type) {
 		return new DfsObjectToPack(objectId, type);
 	}
 
 	private static final Comparator<DfsObjectToPack> OFFSET_SORT = new Comparator<DfsObjectToPack>() {
+		@Override
 		public int compare(DfsObjectToPack a, DfsObjectToPack b) {
 			return Long.signum(a.getOffset() - b.getOffset());
 		}
 	};
 
+	@Override
 	public void selectObjectRepresentation(PackWriter packer,
 			ProgressMonitor monitor, Iterable<ObjectToPack> objects)
 			throws IOException, MissingObjectException {
-		for (DfsPackFile pack : db.getPacks()) {
-			List<DfsObjectToPack> tmp = findAllFromPack(pack, objects);
+		// Don't check dirty bit on PackList; assume ObjectToPacks all came
+		// from the current list.
+		List<DfsPackFile> packs = sortPacksForSelectRepresentation();
+		trySelectRepresentation(packer, monitor, objects, packs, false);
+
+		List<DfsPackFile> garbage = garbagePacksForSelectRepresentation();
+		if (!garbage.isEmpty() && checkGarbagePacks(objects)) {
+			trySelectRepresentation(packer, monitor, objects, garbage, true);
+		}
+	}
+
+	private void trySelectRepresentation(PackWriter packer,
+			ProgressMonitor monitor, Iterable<ObjectToPack> objects,
+			List<DfsPackFile> packs, boolean skipFound) throws IOException {
+		for (DfsPackFile pack : packs) {
+			List<DfsObjectToPack> tmp = findAllFromPack(pack, objects, skipFound);
 			if (tmp.isEmpty())
 				continue;
 			Collections.sort(tmp, OFFSET_SORT);
@@ -471,20 +611,82 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		}
 	}
 
-	private List<DfsObjectToPack> findAllFromPack(DfsPackFile pack,
-			Iterable<ObjectToPack> objects) throws IOException {
-		List<DfsObjectToPack> tmp = new BlockList<DfsObjectToPack>();
-		PackIndex idx = pack.getPackIndex(this);
+	private static final Comparator<DfsPackFile> PACK_SORT_FOR_REUSE = new Comparator<DfsPackFile>() {
+		@Override
+		public int compare(DfsPackFile af, DfsPackFile bf) {
+			DfsPackDescription ad = af.getPackDescription();
+			DfsPackDescription bd = bf.getPackDescription();
+			PackSource as = ad.getPackSource();
+			PackSource bs = bd.getPackSource();
+
+			if (as != null && as == bs && DfsPackDescription.isGC(as)) {
+				// Push smaller GC files last; these likely have higher quality
+				// delta compression and the contained representation should be
+				// favored over other files.
+				return Long.signum(bd.getFileSize(PACK) - ad.getFileSize(PACK));
+			}
+
+			// DfsPackDescription.compareTo already did a reasonable sort.
+			// Rely on Arrays.sort being stable, leaving equal elements.
+			return 0;
+		}
+	};
+
+	private List<DfsPackFile> sortPacksForSelectRepresentation()
+			throws IOException {
+		DfsPackFile[] packs = db.getPacks();
+		List<DfsPackFile> sorted = new ArrayList<>(packs.length);
+		for (DfsPackFile p : packs) {
+			if (p.getPackDescription().getPackSource() != UNREACHABLE_GARBAGE) {
+				sorted.add(p);
+			}
+		}
+		Collections.sort(sorted, PACK_SORT_FOR_REUSE);
+		return sorted;
+	}
+
+	private List<DfsPackFile> garbagePacksForSelectRepresentation()
+			throws IOException {
+		DfsPackFile[] packs = db.getPacks();
+		List<DfsPackFile> garbage = new ArrayList<>(packs.length);
+		for (DfsPackFile p : packs) {
+			if (p.getPackDescription().getPackSource() == UNREACHABLE_GARBAGE) {
+				garbage.add(p);
+			}
+		}
+		return garbage;
+	}
+
+	private static boolean checkGarbagePacks(Iterable<ObjectToPack> objects) {
 		for (ObjectToPack otp : objects) {
+			if (!((DfsObjectToPack) otp).isFound()) {
+				return true;
+			}
+		}
+		return false;
+	}
+
+	private List<DfsObjectToPack> findAllFromPack(DfsPackFile pack,
+			Iterable<ObjectToPack> objects, boolean skipFound)
+					throws IOException {
+		List<DfsObjectToPack> tmp = new BlockList<>();
+		PackIndex idx = pack.getPackIndex(this);
+		for (ObjectToPack obj : objects) {
+			DfsObjectToPack otp = (DfsObjectToPack) obj;
+			if (skipFound && otp.isFound()) {
+				continue;
+			}
 			long p = idx.findOffset(otp);
-			if (0 < p) {
+			if (0 < p && !pack.isCorrupt(p)) {
 				otp.setOffset(p);
-				tmp.add((DfsObjectToPack) otp);
+				tmp.add(otp);
 			}
 		}
 		return tmp;
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public void copyObjectAsIs(PackOutputStream out, ObjectToPack otp,
 			boolean validate) throws IOException,
 			StoredObjectRepresentationNotAvailableException {
@@ -492,21 +694,25 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		src.pack.copyAsIs(out, src, validate, this);
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public void writeObjects(PackOutputStream out, List<ObjectToPack> list)
 			throws IOException {
 		for (ObjectToPack otp : list)
 			out.writeObject(otp);
 	}
 
-	public void copyPackAsIs(PackOutputStream out, CachedPack pack,
-			boolean validate) throws IOException {
-		((DfsCachedPack) pack).copyAsIs(out, validate, this);
+	/** {@inheritDoc} */
+	@Override
+	public void copyPackAsIs(PackOutputStream out, CachedPack pack)
+			throws IOException {
+		((DfsCachedPack) pack).copyAsIs(out, this);
 	}
 
 	/**
 	 * Copy bytes from the window to a caller supplied buffer.
 	 *
-	 * @param pack
+	 * @param file
 	 *            the file the desired window is stored within.
 	 * @param position
 	 *            position within the file to read from.
@@ -525,74 +731,28 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 	 *             this cursor does not match the provider or id and the proper
 	 *             window could not be acquired through the provider's cache.
 	 */
-	int copy(DfsPackFile pack, long position, byte[] dstbuf, int dstoff, int cnt)
-			throws IOException {
+	int copy(BlockBasedFile file, long position, byte[] dstbuf, int dstoff,
+			int cnt) throws IOException {
 		if (cnt == 0)
 			return 0;
 
-		long length = pack.length;
+		long length = file.length;
 		if (0 <= length && length <= position)
 			return 0;
 
 		int need = cnt;
 		do {
-			pin(pack, position);
+			pin(file, position);
 			int r = block.copy(position, dstbuf, dstoff, need);
 			position += r;
 			dstoff += r;
 			need -= r;
 			if (length < 0)
-				length = pack.length;
+				length = file.length;
 		} while (0 < need && position < length);
 		return cnt - need;
 	}
 
-	void copyPackAsIs(DfsPackFile pack, long length, boolean validate,
-			PackOutputStream out) throws IOException {
-		MessageDigest md = null;
-		if (validate) {
-			md = Constants.newMessageDigest();
-			byte[] buf = out.getCopyBuffer();
-			pin(pack, 0);
-			if (block.copy(0, buf, 0, 12) != 12) {
-				pack.setInvalid();
-				throw new IOException(MessageFormat.format(
-						JGitText.get().packfileIsTruncated, pack.getPackName()));
-			}
-			md.update(buf, 0, 12);
-		}
-
-		long position = 12;
-		long remaining = length - (12 + 20);
-		while (0 < remaining) {
-			pin(pack, position);
-
-			int ptr = (int) (position - block.start);
-			int n = (int) Math.min(block.size() - ptr, remaining);
-			block.write(out, position, n, md);
-			position += n;
-			remaining -= n;
-		}
-
-		if (md != null) {
-			byte[] buf = new byte[20];
-			byte[] actHash = md.digest();
-
-			pin(pack, position);
-			if (block.copy(position, buf, 0, 20) != 20) {
-				pack.setInvalid();
-				throw new IOException(MessageFormat.format(
-						JGitText.get().packfileIsTruncated, pack.getPackName()));
-			}
-			if (!Arrays.equals(actHash, buf)) {
-				pack.setInvalid();
-				throw new IOException(MessageFormat.format(
-						JGitText.get().packfileCorruptionDetected,
-						pack.getPackDescription().getFileName(PACK)));
-			}
-		}
-	}
-
 	/**
 	 * Inflate a region of the pack starting at {@code position}.
 	 *
@@ -622,9 +782,10 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 		for (int dstoff = 0;;) {
 			int n = inf.inflate(dstbuf, dstoff, dstbuf.length - dstoff);
 			dstoff += n;
-			if (inf.finished() || (headerOnly && dstoff == dstbuf.length))
+			if (inf.finished() || (headerOnly && dstoff == dstbuf.length)) {
+				stats.inflatedBytes += dstoff;
 				return dstoff;
-			if (inf.needsInput()) {
+			} else if (inf.needsInput()) {
 				pin(pack, position);
 				position += block.setInput(position, inf);
 			} else if (n == 0)
@@ -652,21 +813,37 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 			inf.reset();
 	}
 
-	void pin(DfsPackFile pack, long position) throws IOException {
-		DfsBlock b = block;
-		if (b == null || !b.contains(pack.key, position)) {
+	void pin(BlockBasedFile file, long position) throws IOException {
+		if (block == null || !block.contains(file.key, position)) {
 			// If memory is low, we may need what is in our window field to
 			// be cleaned up by the GC during the get for the next window.
 			// So we always clear it, even though we are just going to set
 			// it again.
 			block = null;
-			block = pack.getOrLoadBlock(position, this);
+			block = file.getOrLoadBlock(position, this);
 		}
 	}
 
-	/** Release the current window cursor. */
+	void unpin() {
+		block = null;
+	}
+
+	/**
+	 * Get IO statistics accumulated by this reader.
+	 *
+	 * @return IO statistics accumulated by this reader.
+	 */
+	public DfsReaderIoStats getIoStats() {
+		return new DfsReaderIoStats(stats);
+	}
+
+	/**
+	 * {@inheritDoc}
+	 * <p>
+	 * Release the current window cursor.
+	 */
 	@Override
-	public void release() {
+	public void close() {
 		last = null;
 		block = null;
 		baseCache = null;
