@@ -51,7 +51,9 @@ import java.util.Date;
 import java.util.Locale;
 import java.util.TimeZone;
 
+import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.util.SystemReader;
+import org.eclipse.jgit.util.time.ProposedTimestamp;
 
 /**
  * A combination of a person identity and time in Git.
@@ -62,210 +64,31 @@ import org.eclipse.jgit.util.SystemReader;
 public class PersonIdent implements Serializable {
 	private static final long serialVersionUID = 1L;
 
-	private final String name;
-
-	private final String emailAddress;
-
-	private final long when;
-
-	private final int tzOffset;
-
 	/**
-	 * Creates new PersonIdent from config info in repository, with current time.
-	 * This new PersonIdent gets the info from the default committer as available
-	 * from the configuration.
+	 * Get timezone object for the given offset.
 	 *
-	 * @param repo
+	 * @param tzOffset
+	 *            timezone offset as in {@link #getTimeZoneOffset()}.
+	 * @return time zone object for the given offset.
+	 * @since 4.1
 	 */
-	public PersonIdent(final Repository repo) {
-		this(repo.getConfig().get(UserConfig.KEY));
-	}
-
-	/**
-	 * Copy a {@link PersonIdent}.
-	 *
-	 * @param pi
-	 *            Original {@link PersonIdent}
-	 */
-	public PersonIdent(final PersonIdent pi) {
-		this(pi.getName(), pi.getEmailAddress());
-	}
-
-	/**
-	 * Construct a new {@link PersonIdent} with current time.
-	 *
-	 * @param aName
-	 * @param aEmailAddress
-	 */
-	public PersonIdent(final String aName, final String aEmailAddress) {
-		this(aName, aEmailAddress, SystemReader.getInstance().getCurrentTime());
-	}
-
-	/**
-	 * Copy a PersonIdent, but alter the clone's time stamp
-	 *
-	 * @param pi
-	 *            original {@link PersonIdent}
-	 * @param when
-	 *            local time
-	 * @param tz
-	 *            time zone
-	 */
-	public PersonIdent(final PersonIdent pi, final Date when, final TimeZone tz) {
-		this(pi.getName(), pi.getEmailAddress(), when, tz);
-	}
-
-	/**
-	 * Copy a {@link PersonIdent}, but alter the clone's time stamp
-	 *
-	 * @param pi
-	 *            original {@link PersonIdent}
-	 * @param aWhen
-	 *            local time
-	 */
-	public PersonIdent(final PersonIdent pi, final Date aWhen) {
-		this(pi.getName(), pi.getEmailAddress(), aWhen.getTime(), pi.tzOffset);
-	}
-
-	/**
-	 * Construct a PersonIdent from simple data
-	 *
-	 * @param aName
-	 * @param aEmailAddress
-	 * @param aWhen
-	 *            local time stamp
-	 * @param aTZ
-	 *            time zone
-	 */
-	public PersonIdent(final String aName, final String aEmailAddress,
-			final Date aWhen, final TimeZone aTZ) {
-		this(aName, aEmailAddress, aWhen.getTime(), aTZ.getOffset(aWhen
-				.getTime()) / (60 * 1000));
-	}
-
-	/**
-	 * Copy a PersonIdent, but alter the clone's time stamp
-	 *
-	 * @param pi
-	 *            original {@link PersonIdent}
-	 * @param aWhen
-	 *            local time stamp
-	 * @param aTZ
-	 *            time zone
-	 */
-	public PersonIdent(final PersonIdent pi, final long aWhen, final int aTZ) {
-		this(pi.getName(), pi.getEmailAddress(), aWhen, aTZ);
-	}
-
-	private PersonIdent(final String aName, final String aEmailAddress,
-			long when) {
-		this(aName, aEmailAddress, when, SystemReader.getInstance()
-				.getTimezone(when));
-	}
-
-	private PersonIdent(final UserConfig config) {
-		this(config.getCommitterName(), config.getCommitterEmail());
-	}
-
-	/**
-	 * Construct a {@link PersonIdent}
-	 *
-	 * @param aName
-	 * @param aEmailAddress
-	 * @param aWhen
-	 *            local time stamp
-	 * @param aTZ
-	 *            time zone
-	 */
-	public PersonIdent(final String aName, final String aEmailAddress,
-			final long aWhen, final int aTZ) {
-		if (aName == null)
-			throw new IllegalArgumentException(
-					"Name of PersonIdent must not be null.");
-		if (aEmailAddress == null)
-			throw new IllegalArgumentException(
-					"E-mail address of PersonIdent must not be null.");
-		name = aName;
-		emailAddress = aEmailAddress;
-		when = aWhen;
-		tzOffset = aTZ;
-	}
-
-	/**
-	 * @return Name of person
-	 */
-	public String getName() {
-		return name;
-	}
-
-	/**
-	 * @return email address of person
-	 */
-	public String getEmailAddress() {
-		return emailAddress;
-	}
-
-	/**
-	 * @return timestamp
-	 */
-	public Date getWhen() {
-		return new Date(when);
-	}
-
-	/**
-	 * @return this person's declared time zone; null if time zone is unknown.
-	 */
-	public TimeZone getTimeZone() {
+	public static TimeZone getTimeZone(int tzOffset) {
 		StringBuilder tzId = new StringBuilder(8);
 		tzId.append("GMT"); //$NON-NLS-1$
-		appendTimezone(tzId);
+		appendTimezone(tzId, tzOffset);
 		return TimeZone.getTimeZone(tzId.toString());
 	}
 
 	/**
-	 * @return this person's declared time zone as minutes east of UTC. If the
-	 *         timezone is to the west of UTC it is negative.
-	 */
-	public int getTimeZoneOffset() {
-		return tzOffset;
-	}
-
-	public int hashCode() {
-		int hc = getEmailAddress().hashCode();
-		hc *= 31;
-		hc += (int) (when / 1000L);
-		return hc;
-	}
-
-	public boolean equals(final Object o) {
-		if (o instanceof PersonIdent) {
-			final PersonIdent p = (PersonIdent) o;
-			return getName().equals(p.getName())
-					&& getEmailAddress().equals(p.getEmailAddress())
-					&& when / 1000L == p.when / 1000L;
-		}
-		return false;
-	}
-
-	/**
-	 * Format for Git storage.
+	 * Format a timezone offset.
 	 *
-	 * @return a string in the git author format
+	 * @param r
+	 *            string builder to append to.
+	 * @param offset
+	 *            timezone offset as in {@link #getTimeZoneOffset()}.
+	 * @since 4.1
 	 */
-	public String toExternalString() {
-		final StringBuilder r = new StringBuilder();
-		r.append(getName().trim());
-		r.append(" <"); //$NON-NLS-1$
-		r.append(getEmailAddress().trim());
-		r.append("> "); //$NON-NLS-1$
-		r.append(when / 1000);
-		r.append(' ');
-		appendTimezone(r);
-		return r.toString();
-	}
-
-	private void appendTimezone(final StringBuilder r) {
-		int offset = tzOffset;
+	public static void appendTimezone(StringBuilder r, int offset) {
 		final char sign;
 		final int offsetHours;
 		final int offsetMins;
@@ -291,6 +114,291 @@ public class PersonIdent implements Serializable {
 		r.append(offsetMins);
 	}
 
+	/**
+	 * Sanitize the given string for use in an identity and append to output.
+	 * <p>
+	 * Trims whitespace from both ends and special characters {@code \n < >} that
+	 * interfere with parsing; appends all other characters to the output.
+	 * Analogous to the C git function {@code strbuf_addstr_without_crud}.
+	 *
+	 * @param r
+	 *            string builder to append to.
+	 * @param str
+	 *            input string.
+	 * @since 4.4
+	 */
+	public static void appendSanitized(StringBuilder r, String str) {
+		// Trim any whitespace less than \u0020 as in String#trim().
+		int i = 0;
+		while (i < str.length() && str.charAt(i) <= ' ') {
+			i++;
+		}
+		int end = str.length();
+		while (end > i && str.charAt(end - 1) <= ' ') {
+			end--;
+		}
+
+		for (; i < end; i++) {
+			char c = str.charAt(i);
+			switch (c) {
+				case '\n':
+				case '<':
+				case '>':
+					continue;
+				default:
+					r.append(c);
+					break;
+			}
+		}
+	}
+
+	private final String name;
+
+	private final String emailAddress;
+
+	private final long when;
+
+	private final int tzOffset;
+
+	/**
+	 * Creates new PersonIdent from config info in repository, with current time.
+	 * This new PersonIdent gets the info from the default committer as available
+	 * from the configuration.
+	 *
+	 * @param repo a {@link org.eclipse.jgit.lib.Repository} object.
+	 */
+	public PersonIdent(final Repository repo) {
+		this(repo.getConfig().get(UserConfig.KEY));
+	}
+
+	/**
+	 * Copy a {@link org.eclipse.jgit.lib.PersonIdent}.
+	 *
+	 * @param pi
+	 *            Original {@link org.eclipse.jgit.lib.PersonIdent}
+	 */
+	public PersonIdent(final PersonIdent pi) {
+		this(pi.getName(), pi.getEmailAddress());
+	}
+
+	/**
+	 * Construct a new {@link org.eclipse.jgit.lib.PersonIdent} with current
+	 * time.
+	 *
+	 * @param aName
+	 *            a {@link java.lang.String} object.
+	 * @param aEmailAddress
+	 *            a {@link java.lang.String} object.
+	 */
+	public PersonIdent(final String aName, final String aEmailAddress) {
+		this(aName, aEmailAddress, SystemReader.getInstance().getCurrentTime());
+	}
+
+	/**
+	 * Construct a new {@link org.eclipse.jgit.lib.PersonIdent} with current
+	 * time.
+	 *
+	 * @param aName
+	 *            a {@link java.lang.String} object.
+	 * @param aEmailAddress
+	 *            a {@link java.lang.String} object.
+	 * @param when
+	 *            a {@link org.eclipse.jgit.util.time.ProposedTimestamp} object.
+	 * @since 4.6
+	 */
+	public PersonIdent(String aName, String aEmailAddress,
+			ProposedTimestamp when) {
+		this(aName, aEmailAddress, when.millis());
+	}
+
+	/**
+	 * Copy a PersonIdent, but alter the clone's time stamp
+	 *
+	 * @param pi
+	 *            original {@link org.eclipse.jgit.lib.PersonIdent}
+	 * @param when
+	 *            local time
+	 * @param tz
+	 *            time zone
+	 */
+	public PersonIdent(final PersonIdent pi, final Date when, final TimeZone tz) {
+		this(pi.getName(), pi.getEmailAddress(), when, tz);
+	}
+
+	/**
+	 * Copy a {@link org.eclipse.jgit.lib.PersonIdent}, but alter the clone's
+	 * time stamp
+	 *
+	 * @param pi
+	 *            original {@link org.eclipse.jgit.lib.PersonIdent}
+	 * @param aWhen
+	 *            local time
+	 */
+	public PersonIdent(final PersonIdent pi, final Date aWhen) {
+		this(pi.getName(), pi.getEmailAddress(), aWhen.getTime(), pi.tzOffset);
+	}
+
+	/**
+	 * Construct a PersonIdent from simple data
+	 *
+	 * @param aName a {@link java.lang.String} object.
+	 * @param aEmailAddress a {@link java.lang.String} object.
+	 * @param aWhen
+	 *            local time stamp
+	 * @param aTZ
+	 *            time zone
+	 */
+	public PersonIdent(final String aName, final String aEmailAddress,
+			final Date aWhen, final TimeZone aTZ) {
+		this(aName, aEmailAddress, aWhen.getTime(), aTZ.getOffset(aWhen
+				.getTime()) / (60 * 1000));
+	}
+
+	/**
+	 * Copy a PersonIdent, but alter the clone's time stamp
+	 *
+	 * @param pi
+	 *            original {@link org.eclipse.jgit.lib.PersonIdent}
+	 * @param aWhen
+	 *            local time stamp
+	 * @param aTZ
+	 *            time zone
+	 */
+	public PersonIdent(final PersonIdent pi, final long aWhen, final int aTZ) {
+		this(pi.getName(), pi.getEmailAddress(), aWhen, aTZ);
+	}
+
+	private PersonIdent(final String aName, final String aEmailAddress,
+			long when) {
+		this(aName, aEmailAddress, when, SystemReader.getInstance()
+				.getTimezone(when));
+	}
+
+	private PersonIdent(final UserConfig config) {
+		this(config.getCommitterName(), config.getCommitterEmail());
+	}
+
+	/**
+	 * Construct a {@link org.eclipse.jgit.lib.PersonIdent}.
+	 * <p>
+	 * Whitespace in the name and email is preserved for the lifetime of this
+	 * object, but are trimmed by {@link #toExternalString()}. This means that
+	 * parsing the result of {@link #toExternalString()} may not return an
+	 * equivalent instance.
+	 *
+	 * @param aName
+	 *            a {@link java.lang.String} object.
+	 * @param aEmailAddress
+	 *            a {@link java.lang.String} object.
+	 * @param aWhen
+	 *            local time stamp
+	 * @param aTZ
+	 *            time zone
+	 */
+	public PersonIdent(final String aName, final String aEmailAddress,
+			final long aWhen, final int aTZ) {
+		if (aName == null)
+			throw new IllegalArgumentException(
+					JGitText.get().personIdentNameNonNull);
+		if (aEmailAddress == null)
+			throw new IllegalArgumentException(
+					JGitText.get().personIdentEmailNonNull);
+		name = aName;
+		emailAddress = aEmailAddress;
+		when = aWhen;
+		tzOffset = aTZ;
+	}
+
+	/**
+	 * Get name of person
+	 *
+	 * @return Name of person
+	 */
+	public String getName() {
+		return name;
+	}
+
+	/**
+	 * Get email address of person
+	 *
+	 * @return email address of person
+	 */
+	public String getEmailAddress() {
+		return emailAddress;
+	}
+
+	/**
+	 * Get timestamp
+	 *
+	 * @return timestamp
+	 */
+	public Date getWhen() {
+		return new Date(when);
+	}
+
+	/**
+	 * Get this person's declared time zone
+	 *
+	 * @return this person's declared time zone; null if time zone is unknown.
+	 */
+	public TimeZone getTimeZone() {
+		return getTimeZone(tzOffset);
+	}
+
+	/**
+	 * Get this person's declared time zone as minutes east of UTC.
+	 *
+	 * @return this person's declared time zone as minutes east of UTC. If the
+	 *         timezone is to the west of UTC it is negative.
+	 */
+	public int getTimeZoneOffset() {
+		return tzOffset;
+	}
+
+	/**
+	 * {@inheritDoc}
+	 * <p>
+	 * Hashcode is based only on the email address and timestamp.
+	 */
+	@Override
+	public int hashCode() {
+		int hc = getEmailAddress().hashCode();
+		hc *= 31;
+		hc += (int) (when / 1000L);
+		return hc;
+	}
+
+	/** {@inheritDoc} */
+	@Override
+	public boolean equals(final Object o) {
+		if (o instanceof PersonIdent) {
+			final PersonIdent p = (PersonIdent) o;
+			return getName().equals(p.getName())
+					&& getEmailAddress().equals(p.getEmailAddress())
+					&& when / 1000L == p.when / 1000L;
+		}
+		return false;
+	}
+
+	/**
+	 * Format for Git storage.
+	 *
+	 * @return a string in the git author format
+	 */
+	public String toExternalString() {
+		final StringBuilder r = new StringBuilder();
+		appendSanitized(r, getName());
+		r.append(" <"); //$NON-NLS-1$
+		appendSanitized(r, getEmailAddress());
+		r.append("> "); //$NON-NLS-1$
+		r.append(when / 1000);
+		r.append(' ');
+		appendTimezone(r, tzOffset);
+		return r.toString();
+	}
+
+	/** {@inheritDoc} */
+	@Override
 	@SuppressWarnings("nls")
 	public String toString() {
 		final StringBuilder r = new StringBuilder();
@@ -309,3 +417,4 @@ public class PersonIdent implements Serializable {
 		return r.toString();
 	}
 }
+
