@@ -74,6 +74,20 @@ public class PackConfig {
 	 */
 	public static final boolean DEFAULT_REUSE_OBJECTS = true;
 
+	/**
+	 * Default value of keep old packs option: {@value}
+	 * @see #setPreserveOldPacks(boolean)
+	 * @since 4.7
+	 */
+	public static final boolean DEFAULT_PRESERVE_OLD_PACKS = false;
+
+	/**
+	 * Default value of prune old packs option: {@value}
+	 * @see #setPrunePreserved(boolean)
+	 * @since 4.7
+	 */
+	public static final boolean DEFAULT_PRUNE_PRESERVED = false;
+
 	/**
 	 * Default value of delta compress option: {@value}
 	 *
@@ -138,6 +152,65 @@ public class PackConfig {
 	 */
 	public static final boolean DEFAULT_BUILD_BITMAPS = true;
 
+	/**
+	 * Default count of most recent commits to select for bitmaps. Only applies
+	 * when bitmaps are enabled: {@value}
+	 *
+	 * @see #setBitmapContiguousCommitCount(int)
+	 * @since 4.2
+	 */
+	public static final int DEFAULT_BITMAP_CONTIGUOUS_COMMIT_COUNT = 100;
+
+	/**
+	 * Count at which the span between selected commits changes from
+	 * "bitmapRecentCommitSpan" to "bitmapDistantCommitSpan". Only applies when
+	 * bitmaps are enabled: {@value}
+	 *
+	 * @see #setBitmapRecentCommitCount(int)
+	 * @since 4.2
+	 */
+	public static final int DEFAULT_BITMAP_RECENT_COMMIT_COUNT = 20000;
+
+	/**
+	 * Default spacing between commits in recent history when selecting commits
+	 * for bitmaps. Only applies when bitmaps are enabled: {@value}
+	 *
+	 * @see #setBitmapRecentCommitSpan(int)
+	 * @since 4.2
+	 */
+	public static final int DEFAULT_BITMAP_RECENT_COMMIT_SPAN = 100;
+
+	/**
+	 * Default spacing between commits in distant history when selecting commits
+	 * for bitmaps. Only applies when bitmaps are enabled: {@value}
+	 *
+	 * @see #setBitmapDistantCommitSpan(int)
+	 * @since 4.2
+	 */
+	public static final int DEFAULT_BITMAP_DISTANT_COMMIT_SPAN = 5000;
+
+	/**
+	 * Default count of branches required to activate inactive branch commit
+	 * selection. If the number of branches is less than this then bitmaps for
+	 * the entire commit history of all branches will be created, otherwise
+	 * branches marked as "inactive" will have coverage for only partial
+	 * history: {@value}
+	 *
+	 * @see #setBitmapExcessiveBranchCount(int)
+	 * @since 4.2
+	 */
+	public static final int DEFAULT_BITMAP_EXCESSIVE_BRANCH_COUNT = 100;
+
+	/**
+	 * Default age at which a branch is considered inactive. Age is taken as the
+	 * number of days ago that the most recent commit was made to a branch. Only
+	 * affects bitmap processing if bitmaps are enabled and the
+	 * "excessive branch count" has been exceeded: {@value}
+	 *
+	 * @see #setBitmapInactiveBranchAgeInDays(int)
+	 * @since 4.2
+	 */
+	public static final int DEFAULT_BITMAP_INACTIVE_BRANCH_AGE_IN_DAYS = 90;
 
 	private int compressionLevel = Deflater.DEFAULT_COMPRESSION;
 
@@ -145,6 +218,10 @@ public class PackConfig {
 
 	private boolean reuseObjects = DEFAULT_REUSE_OBJECTS;
 
+	private boolean preserveOldPacks = DEFAULT_PRESERVE_OLD_PACKS;
+
+	private boolean prunePreserved = DEFAULT_PRUNE_PRESERVED;
+
 	private boolean deltaBaseAsOffset = DEFAULT_DELTA_BASE_AS_OFFSET;
 
 	private boolean deltaCompress = DEFAULT_DELTA_COMPRESS;
@@ -169,9 +246,25 @@ public class PackConfig {
 
 	private boolean buildBitmaps = DEFAULT_BUILD_BITMAPS;
 
+	private int bitmapContiguousCommitCount = DEFAULT_BITMAP_CONTIGUOUS_COMMIT_COUNT;
+
+	private int bitmapRecentCommitCount = DEFAULT_BITMAP_RECENT_COMMIT_COUNT;
+
+	private int bitmapRecentCommitSpan = DEFAULT_BITMAP_RECENT_COMMIT_SPAN;
+
+	private int bitmapDistantCommitSpan = DEFAULT_BITMAP_DISTANT_COMMIT_SPAN;
+
+	private int bitmapExcessiveBranchCount = DEFAULT_BITMAP_EXCESSIVE_BRANCH_COUNT;
+
+	private int bitmapInactiveBranchAgeInDays = DEFAULT_BITMAP_INACTIVE_BRANCH_AGE_IN_DAYS;
+
 	private boolean cutDeltaChains;
 
-	/** Create a default configuration. */
+	private boolean singlePack;
+
+	/**
+	 * Create a default configuration.
+	 */
 	public PackConfig() {
 		// Fields are initialized to defaults.
 	}
@@ -189,7 +282,8 @@ public class PackConfig {
 	}
 
 	/**
-	 * Create a configuration honoring settings in a {@link Config}.
+	 * Create a configuration honoring settings in a
+	 * {@link org.eclipse.jgit.lib.Config}.
 	 *
 	 * @param cfg
 	 *            the source to read settings from. The source is not retained
@@ -210,6 +304,8 @@ public class PackConfig {
 		this.compressionLevel = cfg.compressionLevel;
 		this.reuseDeltas = cfg.reuseDeltas;
 		this.reuseObjects = cfg.reuseObjects;
+		this.preserveOldPacks = cfg.preserveOldPacks;
+		this.prunePreserved = cfg.prunePreserved;
 		this.deltaBaseAsOffset = cfg.deltaBaseAsOffset;
 		this.deltaCompress = cfg.deltaCompress;
 		this.maxDeltaDepth = cfg.maxDeltaDepth;
@@ -222,7 +318,14 @@ public class PackConfig {
 		this.executor = cfg.executor;
 		this.indexVersion = cfg.indexVersion;
 		this.buildBitmaps = cfg.buildBitmaps;
+		this.bitmapContiguousCommitCount = cfg.bitmapContiguousCommitCount;
+		this.bitmapRecentCommitCount = cfg.bitmapRecentCommitCount;
+		this.bitmapRecentCommitSpan = cfg.bitmapRecentCommitSpan;
+		this.bitmapDistantCommitSpan = cfg.bitmapDistantCommitSpan;
+		this.bitmapExcessiveBranchCount = cfg.bitmapExcessiveBranchCount;
+		this.bitmapInactiveBranchAgeInDays = cfg.bitmapInactiveBranchAgeInDays;
 		this.cutDeltaChains = cfg.cutDeltaChains;
+		this.singlePack = cfg.singlePack;
 	}
 
 	/**
@@ -286,6 +389,61 @@ public class PackConfig {
 		this.reuseObjects = reuseObjects;
 	}
 
+	/**
+	 * Checks whether to preserve old packs in a preserved directory
+	 *
+	 * Default setting: {@value #DEFAULT_PRESERVE_OLD_PACKS}
+	 *
+	 * @return true if repacking will preserve old pack files.
+	 * @since 4.7
+	 */
+	public boolean isPreserveOldPacks() {
+		return preserveOldPacks;
+	}
+
+	/**
+	 * Set preserve old packs configuration option for repacking.
+	 *
+	 * If enabled, old pack files are moved into a preserved subdirectory instead
+	 * of being deleted
+	 *
+	 * Default setting: {@value #DEFAULT_PRESERVE_OLD_PACKS}
+	 *
+	 * @param preserveOldPacks
+	 *            boolean indicating whether or not preserve old pack files
+	 * @since 4.7
+	 */
+	public void setPreserveOldPacks(boolean preserveOldPacks) {
+		this.preserveOldPacks = preserveOldPacks;
+	}
+
+	/**
+	 * Checks whether to remove preserved pack files in a preserved directory
+	 *
+	 * Default setting: {@value #DEFAULT_PRUNE_PRESERVED}
+	 *
+	 * @return true if repacking will remove preserved pack files.
+	 * @since 4.7
+	 */
+	public boolean isPrunePreserved() {
+		return prunePreserved;
+	}
+
+	/**
+	 * Set prune preserved configuration option for repacking.
+	 *
+	 * If enabled, preserved pack files are removed from a preserved subdirectory
+	 *
+	 * Default setting: {@value #DEFAULT_PRESERVE_OLD_PACKS}
+	 *
+	 * @param prunePreserved
+	 *            boolean indicating whether or not preserve old pack files
+	 * @since 4.7
+	 */
+	public void setPrunePreserved(boolean prunePreserved) {
+		this.prunePreserved = prunePreserved;
+	}
+
 	/**
 	 * True if writer can use offsets to point to a delta base.
 	 *
@@ -376,6 +534,9 @@ public class PackConfig {
 	}
 
 	/**
+	 * Whether existing delta chains should be cut at
+	 * {@link #getMaxDeltaDepth()}.
+	 *
 	 * @return true if existing delta chains should be cut at
 	 *         {@link #getMaxDeltaDepth()}. Default is false, allowing existing
 	 *         chains to be of any length.
@@ -402,6 +563,32 @@ public class PackConfig {
 		cutDeltaChains = cut;
 	}
 
+	/**
+	 * Whether all of refs/* should be packed in a single pack.
+	 *
+	 * @return true if all of refs/* should be packed in a single pack. Default
+	 *         is false, packing a separate GC_REST pack for references outside
+	 *         of refs/heads/* and refs/tags/*.
+	 * @since 4.9
+	 */
+	public boolean getSinglePack() {
+		return singlePack;
+	}
+
+	/**
+	 * If {@code true}, packs a single GC pack for all objects reachable from
+	 * refs/*. Otherwise packs the GC pack with objects reachable from
+	 * refs/heads/* and refs/tags/*, and a GC_REST pack with the remaining
+	 * reachable objects. Disabled by default, packing GC and GC_REST.
+	 *
+	 * @param single
+	 *            true to pack a single GC pack rather than GC and GC_REST packs
+	 * @since 4.9
+	 */
+	public void setSinglePack(boolean single) {
+		singlePack = single;
+	}
+
 	/**
 	 * Get the number of objects to try when looking for a delta base.
 	 *
@@ -599,14 +786,18 @@ public class PackConfig {
 	 * Default setting: 0 (auto-detect processors)
 	 *
 	 * @param threads
-	 *            number of threads to use. If <= 0 the number of available
+	 *            number of threads to use. If &lt;= 0 the number of available
 	 *            processors for this JVM is used.
 	 */
 	public void setThreads(int threads) {
 		this.threads = threads;
 	}
 
-	/** @return the preferred thread pool to execute delta search on. */
+	/**
+	 * Get the preferred thread pool to execute delta search on.
+	 *
+	 * @return the preferred thread pool to execute delta search on.
+	 */
 	public Executor getExecutor() {
 		return executor;
 	}
@@ -650,7 +841,7 @@ public class PackConfig {
 	 *            oldest (most compatible) format available for the objects.
 	 * @see PackIndexWriter
 	 */
-	public void setIndexVersion(final int version) {
+	public void setIndexVersion(int version) {
 		indexVersion = version;
 	}
 
@@ -683,6 +874,162 @@ public class PackConfig {
 		this.buildBitmaps = buildBitmaps;
 	}
 
+	/**
+	 * Get the count of most recent commits for which to build bitmaps.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_CONTIGUOUS_COMMIT_COUNT}
+	 *
+	 * @return the count of most recent commits for which to build bitmaps
+	 * @since 4.2
+	 */
+	public int getBitmapContiguousCommitCount() {
+		return bitmapContiguousCommitCount;
+	}
+
+	/**
+	 * Set the count of most recent commits for which to build bitmaps.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_CONTIGUOUS_COMMIT_COUNT}
+	 *
+	 * @param count
+	 *            the count of most recent commits for which to build bitmaps
+	 * @since 4.2
+	 */
+	public void setBitmapContiguousCommitCount(int count) {
+		bitmapContiguousCommitCount = count;
+	}
+
+	/**
+	 * Get the count at which to switch from "bitmapRecentCommitSpan" to
+	 * "bitmapDistantCommitSpan".
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_RECENT_COMMIT_COUNT}
+	 *
+	 * @return the count for switching between recent and distant spans
+	 * @since 4.2
+	 */
+	public int getBitmapRecentCommitCount() {
+		return bitmapRecentCommitCount;
+	}
+
+	/**
+	 * Set the count at which to switch from "bitmapRecentCommitSpan" to
+	 * "bitmapDistantCommitSpan".
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_RECENT_COMMIT_COUNT}
+	 *
+	 * @param count
+	 *            the count for switching between recent and distant spans
+	 * @since 4.2
+	 */
+	public void setBitmapRecentCommitCount(int count) {
+		bitmapRecentCommitCount = count;
+	}
+
+	/**
+	 * Get the span of commits when building bitmaps for recent history.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_RECENT_COMMIT_SPAN}
+	 *
+	 * @return the span of commits when building bitmaps for recent history
+	 * @since 4.2
+	 */
+	public int getBitmapRecentCommitSpan() {
+		return bitmapRecentCommitSpan;
+	}
+
+	/**
+	 * Set the span of commits when building bitmaps for recent history.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_RECENT_COMMIT_SPAN}
+	 *
+	 * @param span
+	 *            the span of commits when building bitmaps for recent history
+	 * @since 4.2
+	 */
+	public void setBitmapRecentCommitSpan(int span) {
+		bitmapRecentCommitSpan = span;
+	}
+
+	/**
+	 * Get the span of commits when building bitmaps for distant history.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_DISTANT_COMMIT_SPAN}
+	 *
+	 * @return the span of commits when building bitmaps for distant history
+	 * @since 4.2
+	 */
+	public int getBitmapDistantCommitSpan() {
+		return bitmapDistantCommitSpan;
+	}
+
+	/**
+	 * Set the span of commits when building bitmaps for distant history.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_DISTANT_COMMIT_SPAN}
+	 *
+	 * @param span
+	 *            the span of commits when building bitmaps for distant history
+	 * @since 4.2
+	 */
+	public void setBitmapDistantCommitSpan(int span) {
+		bitmapDistantCommitSpan = span;
+	}
+
+	/**
+	 * Get the count of branches deemed "excessive". If the count of branches in
+	 * a repository exceeds this number and bitmaps are enabled, "inactive"
+	 * branches will have fewer bitmaps than "active" branches.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_EXCESSIVE_BRANCH_COUNT}
+	 *
+	 * @return the count of branches deemed "excessive"
+	 * @since 4.2
+	 */
+	public int getBitmapExcessiveBranchCount() {
+		return bitmapExcessiveBranchCount;
+	}
+
+	/**
+	 * Set the count of branches deemed "excessive". If the count of branches in
+	 * a repository exceeds this number and bitmaps are enabled, "inactive"
+	 * branches will have fewer bitmaps than "active" branches.
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_EXCESSIVE_BRANCH_COUNT}
+	 *
+	 * @param count
+	 *            the count of branches deemed "excessive"
+	 * @since 4.2
+	 */
+	public void setBitmapExcessiveBranchCount(int count) {
+		bitmapExcessiveBranchCount = count;
+	}
+
+	/**
+	 * Get the the age in days that marks a branch as "inactive".
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_INACTIVE_BRANCH_AGE_IN_DAYS}
+	 *
+	 * @return the age in days that marks a branch as "inactive"
+	 * @since 4.2
+	 */
+	public int getBitmapInactiveBranchAgeInDays() {
+		return bitmapInactiveBranchAgeInDays;
+	}
+
+	/**
+	 * Set the the age in days that marks a branch as "inactive".
+	 *
+	 * Default setting: {@value #DEFAULT_BITMAP_INACTIVE_BRANCH_AGE_IN_DAYS}
+	 *
+	 * @param ageInDays
+	 *            the age in days that marks a branch as "inactive"
+	 * @since 4.2
+	 */
+	public void setBitmapInactiveBranchAgeInDays(int ageInDays) {
+		bitmapInactiveBranchAgeInDays = ageInDays;
+	}
+
 	/**
 	 * Update properties by setting fields from the configuration.
 	 *
@@ -712,19 +1059,40 @@ public class PackConfig {
 		// These variables aren't standardized
 		//
 		setReuseDeltas(rc.getBoolean("pack", "reusedeltas", isReuseDeltas())); //$NON-NLS-1$ //$NON-NLS-2$
-		setReuseObjects(rc.getBoolean("pack", "reuseobjects", isReuseObjects())); //$NON-NLS-1$ //$NON-NLS-2$
-		setDeltaCompress(rc.getBoolean(
-				"pack", "deltacompression", isDeltaCompress())); //$NON-NLS-1$ //$NON-NLS-2$
-		setCutDeltaChains(rc.getBoolean(
-				"pack", "cutdeltachains", getCutDeltaChains())); //$NON-NLS-1$ //$NON-NLS-2$
-		setBuildBitmaps(rc.getBoolean("pack", "buildbitmaps", isBuildBitmaps())); //$NON-NLS-1$ //$NON-NLS-2$
+		setReuseObjects(
+				rc.getBoolean("pack", "reuseobjects", isReuseObjects())); //$NON-NLS-1$ //$NON-NLS-2$
+		setDeltaCompress(
+				rc.getBoolean("pack", "deltacompression", isDeltaCompress())); //$NON-NLS-1$ //$NON-NLS-2$
+		setCutDeltaChains(
+				rc.getBoolean("pack", "cutdeltachains", getCutDeltaChains())); //$NON-NLS-1$ //$NON-NLS-2$
+		setSinglePack(
+				rc.getBoolean("pack", "singlepack", getSinglePack())); //$NON-NLS-1$ //$NON-NLS-2$
+		setBuildBitmaps(
+				rc.getBoolean("pack", "buildbitmaps", isBuildBitmaps())); //$NON-NLS-1$ //$NON-NLS-2$
+		setBitmapContiguousCommitCount(
+				rc.getInt("pack", "bitmapcontiguouscommitcount", //$NON-NLS-1$ //$NON-NLS-2$
+						getBitmapContiguousCommitCount()));
+		setBitmapRecentCommitCount(rc.getInt("pack", "bitmaprecentcommitcount", //$NON-NLS-1$ //$NON-NLS-2$
+				getBitmapRecentCommitCount()));
+		setBitmapRecentCommitSpan(rc.getInt("pack", "bitmaprecentcommitspan", //$NON-NLS-1$ //$NON-NLS-2$
+				getBitmapRecentCommitSpan()));
+		setBitmapDistantCommitSpan(rc.getInt("pack", "bitmapdistantcommitspan", //$NON-NLS-1$ //$NON-NLS-2$
+				getBitmapDistantCommitSpan()));
+		setBitmapExcessiveBranchCount(rc.getInt("pack", //$NON-NLS-1$
+				"bitmapexcessivebranchcount", getBitmapExcessiveBranchCount())); //$NON-NLS-1$
+		setBitmapInactiveBranchAgeInDays(
+				rc.getInt("pack", "bitmapinactivebranchageindays", //$NON-NLS-1$ //$NON-NLS-2$
+						getBitmapInactiveBranchAgeInDays()));
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public String toString() {
 		final StringBuilder b = new StringBuilder();
 		b.append("maxDeltaDepth=").append(getMaxDeltaDepth()); //$NON-NLS-1$
 		b.append(", deltaSearchWindowSize=").append(getDeltaSearchWindowSize()); //$NON-NLS-1$
-		b.append(", deltaSearchMemoryLimit=").append(getDeltaSearchMemoryLimit()); //$NON-NLS-1$
+		b.append(", deltaSearchMemoryLimit=") //$NON-NLS-1$
+				.append(getDeltaSearchMemoryLimit());
 		b.append(", deltaCacheSize=").append(getDeltaCacheSize()); //$NON-NLS-1$
 		b.append(", deltaCacheLimit=").append(getDeltaCacheLimit()); //$NON-NLS-1$
 		b.append(", compressionLevel=").append(getCompressionLevel()); //$NON-NLS-1$
@@ -735,6 +1103,19 @@ public class PackConfig {
 		b.append(", reuseObjects=").append(isReuseObjects()); //$NON-NLS-1$
 		b.append(", deltaCompress=").append(isDeltaCompress()); //$NON-NLS-1$
 		b.append(", buildBitmaps=").append(isBuildBitmaps()); //$NON-NLS-1$
+		b.append(", bitmapContiguousCommitCount=") //$NON-NLS-1$
+				.append(getBitmapContiguousCommitCount());
+		b.append(", bitmapRecentCommitCount=") //$NON-NLS-1$
+				.append(getBitmapRecentCommitCount());
+		b.append(", bitmapRecentCommitSpan=") //$NON-NLS-1$
+				.append(getBitmapRecentCommitSpan());
+		b.append(", bitmapDistantCommitSpan=") //$NON-NLS-1$
+				.append(getBitmapDistantCommitSpan());
+		b.append(", bitmapExcessiveBranchCount=") //$NON-NLS-1$
+				.append(getBitmapExcessiveBranchCount());
+		b.append(", bitmapInactiveBranchAge=") //$NON-NLS-1$
+				.append(getBitmapInactiveBranchAgeInDays());
+		b.append(", singlePack=").append(getSinglePack()); //$NON-NLS-1$
 		return b.toString();
 	}
 }
