@@ -82,6 +82,30 @@ public class RefSpec implements Serializable {
 	/** Is this specification actually a wildcard match? */
 	private boolean wildcard;
 
+	/**
+	 * How strict to be about wildcards.
+	 *
+	 * @since 4.5
+	 */
+	public enum WildcardMode {
+		/**
+		 * Reject refspecs with an asterisk on the source side and not the
+		 * destination side or vice versa. This is the mode used by FetchCommand
+		 * and PushCommand to create a one-to-one mapping between source and
+		 * destination refs.
+		 */
+		REQUIRE_MATCH,
+		/**
+		 * Allow refspecs with an asterisk on only one side. This can create a
+		 * many-to-one mapping between source and destination refs, so
+		 * expandFromSource and expandFromDestination are not usable in this
+		 * mode.
+		 */
+		ALLOW_MISMATCH
+	}
+	/** Whether a wildcard is allowed on one side but not the other. */
+	private WildcardMode allowMismatchedWildcards;
+
 	/** Name of the ref(s) we would copy from. */
 	private String srcName;
 
@@ -99,6 +123,83 @@ public class RefSpec implements Serializable {
 		wildcard = false;
 		srcName = Constants.HEAD;
 		dstName = null;
+		allowMismatchedWildcards = WildcardMode.REQUIRE_MATCH;
+	}
+
+	/**
+	 * Parse a ref specification for use during transport operations.
+	 * <p>
+	 * Specifications are typically one of the following forms:
+	 * <ul>
+	 * <li><code>refs/heads/master</code></li>
+	 * <li><code>refs/heads/master:refs/remotes/origin/master</code></li>
+	 * <li><code>refs/heads/*:refs/remotes/origin/*</code></li>
+	 * <li><code>+refs/heads/master</code></li>
+	 * <li><code>+refs/heads/master:refs/remotes/origin/master</code></li>
+	 * <li><code>+refs/heads/*:refs/remotes/origin/*</code></li>
+	 * <li><code>+refs/pull/&#42;/head:refs/remotes/origin/pr/*</code></li>
+	 * <li><code>:refs/heads/master</code></li>
+	 * </ul>
+	 *
+	 * If the wildcard mode allows mismatches, then these ref specs are also
+	 * valid:
+	 * <ul>
+	 * <li><code>refs/heads/*</code></li>
+	 * <li><code>refs/heads/*:refs/heads/master</code></li>
+	 * </ul>
+	 *
+	 * @param spec
+	 *            string describing the specification.
+	 * @param mode
+	 *            whether to allow a wildcard on one side without a wildcard on
+	 *            the other.
+	 * @throws java.lang.IllegalArgumentException
+	 *             the specification is invalid.
+	 * @since 4.5
+	 */
+	public RefSpec(String spec, WildcardMode mode) {
+		this.allowMismatchedWildcards = mode;
+		String s = spec;
+		if (s.startsWith("+")) { //$NON-NLS-1$
+			force = true;
+			s = s.substring(1);
+		}
+
+		final int c = s.lastIndexOf(':');
+		if (c == 0) {
+			s = s.substring(1);
+			if (isWildcard(s)) {
+				wildcard = true;
+				if (mode == WildcardMode.REQUIRE_MATCH) {
+					throw new IllegalArgumentException(MessageFormat
+							.format(JGitText.get().invalidWildcards, spec));
+				}
+			}
+			dstName = checkValid(s);
+		} else if (c > 0) {
+			String src = s.substring(0, c);
+			String dst = s.substring(c + 1);
+			if (isWildcard(src) && isWildcard(dst)) {
+				// Both contain wildcard
+				wildcard = true;
+			} else if (isWildcard(src) || isWildcard(dst)) {
+				wildcard = true;
+				if (mode == WildcardMode.REQUIRE_MATCH)
+					throw new IllegalArgumentException(MessageFormat
+							.format(JGitText.get().invalidWildcards, spec));
+			}
+			srcName = checkValid(src);
+			dstName = checkValid(dst);
+		} else {
+			if (isWildcard(s)) {
+				if (mode == WildcardMode.REQUIRE_MATCH) {
+					throw new IllegalArgumentException(MessageFormat
+							.format(JGitText.get().invalidWildcards, spec));
+				}
+				wildcard = true;
+			}
+			srcName = checkValid(s);
+		}
 	}
 
 	/**
@@ -118,40 +219,11 @@ public class RefSpec implements Serializable {
 	 *
 	 * @param spec
 	 *            string describing the specification.
-	 * @throws IllegalArgumentException
+	 * @throws java.lang.IllegalArgumentException
 	 *             the specification is invalid.
 	 */
 	public RefSpec(final String spec) {
-		String s = spec;
-		if (s.startsWith("+")) { //$NON-NLS-1$
-			force = true;
-			s = s.substring(1);
-		}
-
-		final int c = s.lastIndexOf(':');
-		if (c == 0) {
-			s = s.substring(1);
-			if (isWildcard(s))
-				throw new IllegalArgumentException(MessageFormat.format(JGitText.get().invalidWildcards, spec));
-			dstName = checkValid(s);
-		} else if (c > 0) {
-			String src = s.substring(0, c);
-			String dst = s.substring(c + 1);
-			if (isWildcard(src) && isWildcard(dst)) {
-				// Both contain wildcard
-				wildcard = true;
-			} else if (isWildcard(src) || isWildcard(dst)) {
-				// If either source or destination has wildcard, the other one
-				// must have as well.
-				throw new IllegalArgumentException(MessageFormat.format(JGitText.get().invalidWildcards, spec));
-			}
-			srcName = checkValid(src);
-			dstName = checkValid(dst);
-		} else {
-			if (isWildcard(s))
-				throw new IllegalArgumentException(MessageFormat.format(JGitText.get().invalidWildcards, spec));
-			srcName = checkValid(s);
-		}
+		this(spec, WildcardMode.REQUIRE_MATCH);
 	}
 
 	private RefSpec(final RefSpec p) {
@@ -159,6 +231,7 @@ public class RefSpec implements Serializable {
 		wildcard = p.isWildcard();
 		srcName = p.getSource();
 		dstName = p.getDestination();
+		allowMismatchedWildcards = p.allowMismatchedWildcards;
 	}
 
 	/**
@@ -215,7 +288,7 @@ public class RefSpec implements Serializable {
 	 * @param source
 	 *            new value for source in the returned instance.
 	 * @return a new RefSpec with source as specified.
-	 * @throws IllegalStateException
+	 * @throws java.lang.IllegalStateException
 	 *             There is already a destination configured, and the wildcard
 	 *             status of the existing destination disagrees with the
 	 *             wildcard status of the new source.
@@ -254,7 +327,7 @@ public class RefSpec implements Serializable {
 	 * @param destination
 	 *            new value for destination in the returned instance.
 	 * @return a new RefSpec with destination as specified.
-	 * @throws IllegalStateException
+	 * @throws java.lang.IllegalStateException
 	 *             There is already a source configured, and the wildcard status
 	 *             of the existing source disagrees with the wildcard status of
 	 *             the new destination.
@@ -277,7 +350,7 @@ public class RefSpec implements Serializable {
 	 * @param destination
 	 *            new value for destination in the returned instance.
 	 * @return a new RefSpec with destination as specified.
-	 * @throws IllegalArgumentException
+	 * @throws java.lang.IllegalArgumentException
 	 *             The wildcard status of the new source disagrees with the
 	 *             wildcard status of the new destination.
 	 */
@@ -348,8 +421,15 @@ public class RefSpec implements Serializable {
 	 * @return a new specification expanded from provided ref name. Result
 	 *         specification is wildcard if and only if provided ref name is
 	 *         wildcard.
+	 * @throws java.lang.IllegalStateException
+	 *             when the RefSpec was constructed with wildcard mode that
+	 *             doesn't require matching wildcards.
 	 */
 	public RefSpec expandFromSource(final String r) {
+		if (allowMismatchedWildcards != WildcardMode.REQUIRE_MATCH) {
+			throw new IllegalStateException(
+					JGitText.get().invalidExpandWildcard);
+		}
 		return isWildcard() ? new RefSpec(this).expandFromSourceImp(r) : this;
 	}
 
@@ -373,6 +453,9 @@ public class RefSpec implements Serializable {
 	 * @return a new specification expanded from provided ref name. Result
 	 *         specification is wildcard if and only if provided ref name is
 	 *         wildcard.
+	 * @throws java.lang.IllegalStateException
+	 *             when the RefSpec was constructed with wildcard mode that
+	 *             doesn't require matching wildcards.
 	 */
 	public RefSpec expandFromSource(final Ref r) {
 		return expandFromSource(r.getName());
@@ -390,8 +473,15 @@ public class RefSpec implements Serializable {
 	 * @return a new specification expanded from provided ref name. Result
 	 *         specification is wildcard if and only if provided ref name is
 	 *         wildcard.
+	 * @throws java.lang.IllegalStateException
+	 *             when the RefSpec was constructed with wildcard mode that
+	 *             doesn't require matching wildcards.
 	 */
 	public RefSpec expandFromDestination(final String r) {
+		if (allowMismatchedWildcards != WildcardMode.REQUIRE_MATCH) {
+			throw new IllegalStateException(
+					JGitText.get().invalidExpandWildcard);
+		}
 		return isWildcard() ? new RefSpec(this).expandFromDstImp(r) : this;
 	}
 
@@ -414,6 +504,9 @@ public class RefSpec implements Serializable {
 	 * @return a new specification expanded from provided ref name. Result
 	 *         specification is wildcard if and only if provided ref name is
 	 *         wildcard.
+	 * @throws java.lang.IllegalStateException
+	 *             when the RefSpec was constructed with wildcard mode that
+	 *             doesn't require matching wildcards.
 	 */
 	public RefSpec expandFromDestination(final Ref r) {
 		return expandFromDestination(r.getName());
@@ -422,7 +515,7 @@ public class RefSpec implements Serializable {
 	private boolean match(final String name, final String s) {
 		if (s == null)
 			return false;
-		if (isWildcard()) {
+		if (isWildcard(s)) {
 			int wildcardIndex = s.indexOf('*');
 			String prefix = s.substring(0, wildcardIndex);
 			String suffix = s.substring(wildcardIndex + 1);
@@ -453,18 +546,18 @@ public class RefSpec implements Serializable {
 			return false;
 		if (s.contains("//")) //$NON-NLS-1$
 			return false;
+		if (s.endsWith("/")) //$NON-NLS-1$
+			return false;
 		int i = s.indexOf('*');
 		if (i != -1) {
 			if (s.indexOf('*', i + 1) > i)
 				return false;
-			if (i > 0 && s.charAt(i - 1) != '/')
-				return false;
-			if (i < s.length() - 1 && s.charAt(i + 1) != '/')
-				return false;
 		}
 		return true;
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public int hashCode() {
 		int hc = 0;
 		if (getSource() != null)
@@ -474,6 +567,8 @@ public class RefSpec implements Serializable {
 		return hc;
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public boolean equals(final Object obj) {
 		if (!(obj instanceof RefSpec))
 			return false;
@@ -497,6 +592,8 @@ public class RefSpec implements Serializable {
 		return a.equals(b);
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public String toString() {
 		final StringBuilder r = new StringBuilder();
 		if (isForceUpdate())
