@@ -49,13 +49,15 @@ import org.eclipse.jgit.lib.AnyObjectId;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.RefUpdate;
 
-/** Update of a locally stored tracking branch. */
+/**
+ * Update of a locally stored tracking branch.
+ */
 public class TrackingRefUpdate {
 	private final String remoteName;
-	private final String localName;
-	private boolean forceUpdate;
-	private ObjectId oldObjectId;
-	private ObjectId newObjectId;
+	final String localName;
+	boolean forceUpdate;
+	ObjectId oldObjectId;
+	ObjectId newObjectId;
 
 	private RefUpdate.Result result;
 	private ReceiveCommand cmd;
@@ -132,6 +134,8 @@ public class TrackingRefUpdate {
 	}
 
 	/**
+	 * Get this update wrapped by a ReceiveCommand.
+	 *
 	 * @return this update wrapped by a ReceiveCommand.
 	 * @since 3.4
 	 */
@@ -142,7 +146,7 @@ public class TrackingRefUpdate {
 	}
 
 	final class Command extends ReceiveCommand {
-		private Command() {
+		Command() {
 			super(oldObjectId, newObjectId, localName);
 		}
 
@@ -201,6 +205,7 @@ public class TrackingRefUpdate {
 		}
 	}
 
+	/** {@inheritDoc} */
 	@SuppressWarnings("nls")
 	@Override
 	public String toString() {
