@@ -44,6 +44,7 @@
 
 package org.eclipse.jgit.util;
 
+import java.io.BufferedOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -55,7 +56,6 @@ import java.util.ArrayList;
 import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.lib.NullProgressMonitor;
 import org.eclipse.jgit.lib.ProgressMonitor;
-import org.eclipse.jgit.util.io.SafeBufferedOutputStream;
 
 /**
  * A fully buffered output stream.
@@ -68,7 +68,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 	protected static final int DEFAULT_IN_CORE_LIMIT = 1024 * 1024;
 
 	/** Chain of data, if we are still completely in-core; otherwise null. */
-	private ArrayList<Block> blocks;
+	ArrayList<Block> blocks;
 
 	/**
 	 * Maximum number of bytes we will permit storing in memory.
@@ -78,6 +78,9 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 */
 	private int inCoreLimit;
 
+	/** Initial size of block list. */
+	private int initialBlocks;
+
 	/** If {@link #inCoreLimit} has been reached, remainder goes here. */
 	private OutputStream overflow;
 
@@ -86,13 +89,32 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 *
 	 * @param limit
 	 *            maximum number of bytes to store in memory before entering the
-	 *            overflow output path.
+	 *            overflow output path; also used as the estimated size.
 	 */
 	protected TemporaryBuffer(final int limit) {
-		inCoreLimit = limit;
+		this(limit, limit);
+	}
+
+	/**
+	 * Create a new empty temporary buffer.
+	 *
+	 * @param estimatedSize
+	 *            estimated size of storage used, to size the initial list of
+	 *            block pointers.
+	 * @param limit
+	 *            maximum number of bytes to store in memory before entering the
+	 *            overflow output path.
+	 * @since 4.0
+	 */
+	protected TemporaryBuffer(final int estimatedSize, final int limit) {
+		if (estimatedSize > limit)
+			throw new IllegalArgumentException();
+		this.inCoreLimit = limit;
+		this.initialBlocks = (estimatedSize - 1) / Block.SZ + 1;
 		reset();
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public void write(final int b) throws IOException {
 		if (overflow != null) {
@@ -113,6 +135,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 		s.buffer[s.count++] = (byte) b;
 	}
 
+	/** {@inheritDoc} */
 	@Override
 	public void write(final byte[] b, int off, int len) throws IOException {
 		if (overflow == null) {
@@ -141,7 +164,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 	/**
 	 * Dumps the entire buffer into the overflow stream, and flushes it.
 	 *
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             the overflow stream cannot be started, or the buffer contents
 	 *             cannot be written to it, or it failed to flush.
 	 */
@@ -156,7 +179,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 *
 	 * @param in
 	 *            the stream to read from, until EOF is reached.
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             an error occurred reading from the input stream, or while
 	 *             writing to a local temporary file.
 	 */
@@ -206,10 +229,8 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 * The buffer is only complete after {@link #close()} has been invoked.
 	 *
 	 * @return the complete byte array; length matches {@link #length()}.
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             an error occurred reading from a local temporary file
-	 * @throws OutOfMemoryError
-	 *             the buffer cannot fit in memory
 	 */
 	public byte[] toByteArray() throws IOException {
 		final long len = length();
@@ -224,6 +245,33 @@ public abstract class TemporaryBuffer extends OutputStream {
 		return out;
 	}
 
+	/**
+	 * Convert this buffer's contents into a contiguous byte array. If this size
+	 * of the buffer exceeds the limit only return the first {@code limit} bytes
+	 * <p>
+	 * The buffer is only complete after {@link #close()} has been invoked.
+	 *
+	 * @param limit
+	 *            the maximum number of bytes to be returned
+	 * @return the byte array limited to {@code limit} bytes.
+	 * @throws java.io.IOException
+	 *             an error occurred reading from a local temporary file
+	 * @since 4.2
+	 */
+	public byte[] toByteArray(int limit) throws IOException {
+		final long len = Math.min(length(), limit);
+		if (Integer.MAX_VALUE < len)
+			throw new OutOfMemoryError(
+					JGitText.get().lengthExceedsMaximumArraySize);
+		final byte[] out = new byte[(int) len];
+		int outPtr = 0;
+		for (final Block b : blocks) {
+			System.arraycopy(b.buffer, 0, out, outPtr, b.count);
+			outPtr += b.count;
+		}
+		return out;
+	}
+
 	/**
 	 * Send this buffer to an output stream.
 	 * <p>
@@ -236,7 +284,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 *            if not null progress updates are sent here. Caller should
 	 *            initialize the task and the number of work units to <code>
 	 *            {@link #length()}/1024</code>.
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             an error occurred reading from a temporary file on the local
 	 *             system, or writing to the output stream.
 	 */
@@ -258,25 +306,45 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 *
 	 * @return a stream to read from the buffer. The caller must close the
 	 *         stream when it is no longer useful.
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             an error occurred opening the temporary file.
 	 */
 	public InputStream openInputStream() throws IOException {
 		return new BlockInputStream();
 	}
 
-	/** Reset this buffer for reuse, purging all buffered content. */
+	/**
+	 * Same as {@link #openInputStream()} but handling destruction of any
+	 * associated resources automatically when closing the returned stream.
+	 *
+	 * @return an InputStream which will automatically destroy any associated
+	 *         temporary file on {@link #close()}
+	 * @throws IOException
+	 *             in case of an error.
+	 * @since 4.11
+	 */
+	public InputStream openInputStreamWithAutoDestroy() throws IOException {
+		return new BlockInputStream() {
+			@Override
+			public void close() throws IOException {
+				super.close();
+				destroy();
+			}
+		};
+	}
+
+	/**
+	 * Reset this buffer for reuse, purging all buffered content.
+	 */
 	public void reset() {
 		if (overflow != null) {
 			destroy();
 		}
-		if (inCoreLimit < Block.SZ) {
-			blocks = new ArrayList<Block>(1);
-			blocks.add(new Block(inCoreLimit));
-		} else {
-			blocks = new ArrayList<Block>(inCoreLimit / Block.SZ);
-			blocks.add(new Block());
-		}
+		if (blocks != null)
+			blocks.clear();
+		else
+			blocks = new ArrayList<>(initialBlocks);
+		blocks.add(new Block(Math.min(inCoreLimit, Block.SZ)));
 	}
 
 	/**
@@ -284,7 +352,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 	 *
 	 * @return the output stream to receive the buffered content, followed by
 	 *         the remaining output.
-	 * @throws IOException
+	 * @throws java.io.IOException
 	 *             the buffer cannot create the overflow stream.
 	 */
 	protected abstract OutputStream overflow() throws IOException;
@@ -309,10 +377,12 @@ public abstract class TemporaryBuffer extends OutputStream {
 			overflow.write(b.buffer, 0, b.count);
 		blocks = null;
 
-		overflow = new SafeBufferedOutputStream(overflow, Block.SZ);
+		overflow = new BufferedOutputStream(overflow, Block.SZ);
 		overflow.write(last.buffer, 0, last.count);
 	}
 
+	/** {@inheritDoc} */
+	@Override
 	public void close() throws IOException {
 		if (overflow != null) {
 			try {
@@ -323,7 +393,9 @@ public abstract class TemporaryBuffer extends OutputStream {
 		}
 	}
 
-	/** Clear this buffer so it has no data, and cannot be used again. */
+	/**
+	 * Clear this buffer so it has no data, and cannot be used again.
+	 */
 	public void destroy() {
 		blocks = null;
 
@@ -361,29 +433,6 @@ public abstract class TemporaryBuffer extends OutputStream {
 		 */
 		private File onDiskFile;
 
-		/**
-		 * Create a new temporary buffer.
-		 *
-		 * @deprecated Use the {@code File} overload to supply a directory.
-		 */
-		@Deprecated
-		public LocalFile() {
-			this(null, DEFAULT_IN_CORE_LIMIT);
-		}
-
-		/**
-		 * Create a new temporary buffer, limiting memory usage.
-		 *
-		 * @param inCoreLimit
-		 *            maximum number of bytes to store in memory. Storage beyond
-		 *            this limit will use the local file.
-		 * @deprecated Use the {@code File,int} overload to supply a directory.
-		 */
-		@Deprecated
-		public LocalFile(final int inCoreLimit) {
-			this(null, inCoreLimit);
-		}
-
 		/**
 		 * Create a new temporary buffer, limiting memory usage.
 		 *
@@ -414,11 +463,13 @@ public abstract class TemporaryBuffer extends OutputStream {
 			this.directory = directory;
 		}
 
+		@Override
 		protected OutputStream overflow() throws IOException {
 			onDiskFile = File.createTempFile("jgit_", ".buf", directory); //$NON-NLS-1$ //$NON-NLS-2$
-			return new FileOutputStream(onDiskFile);
+			return new BufferedOutputStream(new FileOutputStream(onDiskFile));
 		}
 
+		@Override
 		public long length() {
 			if (onDiskFile == null) {
 				return super.length();
@@ -426,6 +477,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 			return onDiskFile.length();
 		}
 
+		@Override
 		public byte[] toByteArray() throws IOException {
 			if (onDiskFile == null) {
 				return super.toByteArray();
@@ -444,6 +496,7 @@ public abstract class TemporaryBuffer extends OutputStream {
 			return out;
 		}
 
+		@Override
 		public void writeTo(final OutputStream os, ProgressMonitor pm)
 				throws IOException {
 			if (onDiskFile == null) {
@@ -472,6 +525,20 @@ public abstract class TemporaryBuffer extends OutputStream {
 			return new FileInputStream(onDiskFile);
 		}
 
+		@Override
+		public InputStream openInputStreamWithAutoDestroy() throws IOException {
+			if (onDiskFile == null) {
+				return super.openInputStreamWithAutoDestroy();
+			}
+			return new FileInputStream(onDiskFile) {
+				@Override
+				public void close() throws IOException {
+					super.close();
+					destroy();
+				}
+			};
+		}
+
 		@Override
 		public void destroy() {
 			super.destroy();
@@ -498,14 +565,30 @@ public abstract class TemporaryBuffer extends OutputStream {
 		 * Create a new heap buffer with a maximum storage limit.
 		 *
 		 * @param limit
-		 *            maximum number of bytes that can be stored in this buffer.
-		 *            Storing beyond this many will cause an IOException to be
-		 *            thrown during write.
+		 *            maximum number of bytes that can be stored in this buffer;
+		 *            also used as the estimated size. Storing beyond this many
+		 *            will cause an IOException to be thrown during write.
 		 */
 		public Heap(final int limit) {
 			super(limit);
 		}
 
+		/**
+		 * Create a new heap buffer with a maximum storage limit.
+		 *
+		 * @param estimatedSize
+		 *            estimated size of storage used, to size the initial list of
+		 *            block pointers.
+		 * @param limit
+		 *            maximum number of bytes that can be stored in this buffer.
+		 *            Storing beyond this many will cause an IOException to be
+		 *            thrown during write.
+		 * @since 4.0
+		 */
+		public Heap(final int estimatedSize, final int limit) {
+			super(estimatedSize, limit);
+		}
+
 		@Override
 		protected OutputStream overflow() throws IOException {
 			throw new IOException(JGitText.get().inMemoryBufferLimitExceeded);
