@@ -51,12 +51,12 @@ group = 'org.eclipse.jgit'
 artifacts = [group,
              group + '.ant',
              group + '.archive',
-             group + '.console',
-			 group + '.http.apache',
+             group + '.http.apache',
              group + '.http.server',
-             group + '.java7',
              group + '.junit',
              group + '.junit.http',
+             group + '.lfs',
+             group + '.lfs.server',
              group + '.pgm',
              group + '.ui']
 
