@@ -131,6 +131,7 @@ perl -pi~ -e '
 		$seen_version = 1 if (!/<\?xml/ &&
 		s/(version=")[^"]*(")/${1}'"$OSGI_V"'${2}/);
 	}
+	s/(import feature="org\.eclipse\.jgit.*" version=")[^"]*(")/${1}'"$API_V"'${2}/;
 	' org.eclipse.jgit.packaging/org.*.feature/feature.xml
 
 perl -pi~ -e '
@@ -144,17 +145,6 @@ perl -pi~ -e '
 	}
 	' org.eclipse.jgit.packaging/org.*.feature/pom.xml
 
-perl -pi~ -e '
-	if ($ARGV ne $old_argv) {
-		$seen_version = 0;
-		$old_argv = $ARGV;
-	}
-	if ($seen_version < 5) {
-		$seen_version++ if
-		s{<(version)>.*</\1>}{<${1}>'"$POM_V"'</${1}>};
-	}
-	' org.eclipse.jgit.packaging/org.eclipse.jgit.updatesite/pom.xml
-
 perl -pi~ -e '
 	if ($ARGV ne $old_argv) {
 		$seen_version = 0;
